"""
If there is a static/ directory inside of a grokked package, its
contents will be available as static resources under a URL:

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open('http://localhost/@@/grok.ftests.staticdir.simple_fixture/'
  ...              'file.txt')
  >>> print browser.contents
  some text

We use a special name 'static' in page templates to allow easy linking
to resources:

  >>> root = getRootFolder()
  >>> from grok.ftests.staticdir.simple_fixture.ellie import Mammoth
  >>> root[u'ellie'] = Mammoth()
  >>> browser.open('http://localhost/ellie')
  >>> print browser.contents
  <html>
  <body>
  <a href="http://localhost/@@/grok.ftests.staticdir.simple_fixture/file.txt">Some text in a file</a>
  </body>
  </html>

Static also means that page templates will not be interpreted:

  >>> browser.open('http://localhost/@@/grok.ftests.staticdir.simple_fixture/static.pt')
  >>> print browser.contents
  <html>
  <body>
  <h1 tal:content="string:will not be interpreted"/>
  </body>
  </html>

We also support subdirectories for resources:

  >>> browser.open('http://localhost/@@/grok.ftests.staticdir.simple_fixture/subdir/otherfile.txt')
  >>> print browser.contents
  This is yet another file.

"""



"""
Viewing a protected view with insufficient privileges will yield
Unauthorized:

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.open("http://localhost/@@painting")
  Traceback (most recent call last):
  HTTPError: HTTP Error 401: Unauthorized

When we log in (e.g. as a manager), we can access the view just fine:

  >>> from zope.app.securitypolicy.rolepermission import rolePermissionManager
  >>> rolePermissionManager.grantPermissionToRole('grok.ViewPainting',
  ...                                             'zope.Manager')
  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/@@painting")
  >>> print browser.contents
  What a beautiful painting.

A view protected with 'zope.Public' is always accessible:

  >>> browser = Browser()
  >>> browser.open("http://localhost/@@publicnudity")
  >>> print browser.contents
  Everybody can see this.
"""

import grok
import zope.interface

class ViewPainting(grok.Permission):
    grok.name('grok.ViewPainting')

class Painting(grok.View):

    grok.context(zope.interface.Interface)
    grok.require('grok.ViewPainting')

    def render(self):
        return 'What a beautiful painting.'

class PublicNudity(grok.View):

    grok.context(zope.interface.Interface)
    grok.require('zope.Public')

    def render(self):
        return 'Everybody can see this.'



"""
This should fail because ``grok.template`` points to a non-existing
template:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  ConfigurationExecutionError: martian.error.GrokError: View <class 'grok.tests.view.templatenotfound.Painting'>
  has no associated template or 'render' method.
  in:
"""
import grok

class Mammoth(grok.Model):
    pass

class Painting(grok.View):
    grok.template('cavepainting')

# no cavepainting template here



from setuptools import setup, find_packages

setup(
    name='grokdocs',
    install_requires=['docutils',
                      'zope.pagetemplate',
                      'zope.app.renderer',
                      'Pygments'
                      ],
    py_modules = ['grok2html'],
    entry_points="""
    [console_scripts]
    grok2html = grok2html:main
    """
    )



"""

  >>> grok.testing.grok(__name__)

We should find the ``cavepainting`` view for a mammoth:

  >>> manfred = Mammoth()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  >>> view = component.getMultiAdapter((manfred, request), name='cavepainting')
  >>> view()
  'A cave painting of a mammoth'

  >>> view.context is manfred
  True
  >>> view.request is request
  True

Look up a view with a name explicitly set with ``grok.name``:

  >>> view = component.getMultiAdapter((manfred, request), name='meal')
  >>> view()
  'Mammoth burger'

There's no view 'food':

  >>> view = component.getMultiAdapter((manfred, request), name='food')
  Traceback (most recent call last):
    ...
  ComponentLookupError: ((<grok.tests.view.view.Mammoth object at 0x...>, <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>), <InterfaceClass zope.interface.Interface>, 'food')

"""

import grok

class Mammoth(grok.Model):
    pass

class CavePainting(grok.View):

    def render(self):
        return 'A cave painting of a mammoth'

class Food(grok.View):
    """Grok says: ME NO SEE MAMMOTH, ME SEE MEAL!"""
    grok.name('meal')

    def render(self):
        return 'Mammoth burger'



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok directives.
"""

from zope.interface.interfaces import IInterface

from martian.error import GrokImportError
from martian.directive import (OnceDirective,
                               MultipleTimesDirective, BaseTextDirective,
                               SingleValue, SingleTextDirective,
                               MultipleTextDirective,
                               MarkerDirective,
                               InterfaceDirective,
                               InterfaceOrClassDirective,
                               ModuleDirectiveContext,
                               ClassDirectiveContext,
                               ClassOrModuleDirectiveContext)
from martian import util

class GlobalUtilityDirective(MultipleTimesDirective):
    def check_arguments(self, factory, provides=None, name=u'',
                        direct=False):
        if provides is not None and not IInterface.providedBy(provides):
            raise GrokImportError("You can only pass an interface to the "
                                  "provides argument of %s." % self.name)

    def value_factory(self, *args, **kw):
        return GlobalUtilityInfo(*args, **kw)


class GlobalUtilityInfo(object):
    def __init__(self, factory, provides=None, name=u'', direct=None):
        self.factory = factory
        if direct is None:
            direct = util.class_annotation(factory, 'grok.direct', False)
        self.direct = direct

        if provides is None:
            provides = util.class_annotation(factory, 'grok.provides', None)
        self.provides = provides

        if name is u'':
            name = util.class_annotation(factory, 'grok.name', u'')
        self.name = name


class LocalUtilityDirective(MultipleTimesDirective):
    def check_arguments(self, factory, provides=None, name=u'',
                        setup=None, public=False, name_in_container=None):
        if provides is not None and not IInterface.providedBy(provides):
            raise GrokImportError("You can only pass an interface to the "
                                  "provides argument of %s." % self.name)

    def value_factory(self, *args, **kw):
        return LocalUtilityInfo(*args, **kw)


class LocalUtilityInfo(object):
    def __init__(self, factory, provides=None, name=u'',
                 setup=None, public=False, name_in_container=None):
        self.factory = factory
        if provides is None:
            provides = util.class_annotation(factory, 'grok.provides', None)
        self.provides = provides
        self.name = name
        self.setup = setup
        self.public = public
        self.name_in_container = name_in_container


class RequireDirective(BaseTextDirective, SingleValue, MultipleTimesDirective):

    def store(self, frame, value):
        super(RequireDirective, self).store(frame, value)
        values = frame.f_locals.get(self.local_name, [])

        # grok.require can be used both as a class-level directive and
        # as a decorator for methods.  Therefore we return a decorator
        # here, which may be used for methods, or simply ignored when
        # used as a directive.
        def decorator(func):
            permission = values.pop()
            func.__grok_require__ = permission
            return func
        return decorator

class MultiValueOnceDirective(OnceDirective):

    def check_arguments(self, *values):
        pass

    def value_factory(self, *args):
        return args

# Define grok directives
name = SingleTextDirective('grok.name', ClassDirectiveContext())
template = SingleTextDirective('grok.template', ClassDirectiveContext())
context = InterfaceOrClassDirective('grok.context',
                                    ClassOrModuleDirectiveContext())
templatedir = SingleTextDirective('grok.templatedir', ModuleDirectiveContext())
provides = InterfaceDirective('grok.provides', ClassDirectiveContext())
baseclass = MarkerDirective('grok.baseclass', ClassDirectiveContext())
global_utility = GlobalUtilityDirective('grok.global_utility',
                                        ModuleDirectiveContext())
local_utility = LocalUtilityDirective('grok.local_utility',
                                      ClassDirectiveContext())
require = RequireDirective('grok.require', ClassDirectiveContext())
site = InterfaceOrClassDirective('grok.site',
                                 ClassDirectiveContext())
title = SingleTextDirective('grok.title', ClassDirectiveContext())
permissions = MultiValueOnceDirective(
    'grok.permissions', ClassDirectiveContext())
layer = InterfaceOrClassDirective('grok.layer',
                           ClassOrModuleDirectiveContext())
direct = MarkerDirective('grok.direct', ClassDirectiveContext())


##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok test helpers
"""
from zope.configuration.config import ConfigurationMachine
from martian import scan
from grok import zcml

def grok(module_name):
    config = ConfigurationMachine()
    zcml.do_grok('grok.meta', config)
    zcml.do_grok('grok.templatereg', config)
    zcml.do_grok(module_name, config)
    config.execute_actions()

def grok_component(name, component,
                   context=None, module_info=None, templates=None):
    if module_info is None:
        obj_module = getattr(component, '__grok_module__', None)
        if obj_module is None:
            obj_module = getattr(component, '__module__', None)
        module_info = scan.module_info_from_dotted_name(obj_module)

    module = module_info.getModule()
    if context is not None:
        module.__grok_context__ = context
    if templates is not None:
        module.__grok_templates__ = templates
    config = ConfigurationMachine()
    result = zcml.the_multi_grokker.grok(name, component,
                                         module_info=module_info,
                                         config=config)
    config.execute_actions()    
    return result



# -*- coding: UTF-8 -*-
"""
There is a url function that can be imported from grok to determine the
absolute URL of objects.

  >>> from grok import url
  
  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> manfred = Mammoth()
  >>> herd['manfred'] = manfred

Now let's use url on some things::

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/herd/manfred/index")
  >>> print browser.contents
  http://localhost/herd/manfred/index
  >>> browser.open("http://localhost/herd/manfred/another")
  >>> print browser.contents
  http://localhost/herd/manfred/another
  
We get the views manually so we can do a greater variety of url() calls:

  >>> from zope import component
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> index_view = component.getMultiAdapter((manfred, request), name='index')
  >>> url(request, index_view)
  'http://127.0.0.1/herd/manfred/index'
  >>> another_view = component.getMultiAdapter((manfred, request),
  ...                                              name='another')
  >>> url(request, another_view)
  'http://127.0.0.1/herd/manfred/another'

Now let's get a URL for a specific object:

  >>> url(request, manfred)
  'http://127.0.0.1/herd/manfred'

We can get the URL for any object in content-space:

  >>> url(request, herd)
  'http://127.0.0.1/herd'

We can also pass a name along with this, to generate a URL to a
particular view on the object:

  >>> url(request, herd, 'something')
  'http://127.0.0.1/herd/something'

It works properly in the face of non-ascii characters in URLs:

  >>> u = url(request, herd, unicode('árgh', 'UTF-8'))
  >>> u
  'http://127.0.0.1/herd/%C3%A1rgh'
  >>> import urllib
  >>> expected = unicode('http://127.0.0.1/herd/árgh', 'UTF-8')
  >>> urllib.unquote(u).decode('utf-8') == expected
  True
"""
import grok
from grok import url

class Herd(grok.Container, grok.Model):
    pass

class Mammoth(grok.Model):
    pass

grok.context(Mammoth)

class Index(grok.View):
    def render(self):
        return url(self.request, self)
    
class Another(grok.View):
    def render(self):
        return url(self.request, self)



# -*- coding: latin-1 -*-
"""
We can define a few permissions with grok.Permission and then take a
look at them in Zope 3's grant view:

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.open("http://localhost/@@grant.html")

  >>> browser.getControl(name='field.principal.MA__.searchstring').value = 'manager'
  >>> browser.getControl('Search').click()
  >>> browser.getControl('Apply').click()
  >>> 'grok.ascii-permission' in browser.contents
  True

"""
import grok

class ASCIIPermission(grok.Permission):
    grok.name('grok.ascii-permission')

# TODO Technically, it's absolutely possible to give permissions
# non-ASCII names. However the way Zope 3's grant view uses widgets to
# display form controls for each permission is not unicode-safe.



"""
A grok.EditForm is a special grok.View that renders an edit form.

  >>> getRootFolder()["manfred"] = Mammoth()

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/manfred/@@edit")
  >>> browser.getControl(name="form.name").value = "Manfred the Mammoth"
  >>> browser.getControl(name="form.size").value = "Really big"
  >>> browser.getControl("Apply").click()
  >>> print browser.contents
  <html>...
  ...Manfred the Mammoth...
  ...Really big...
  ...

grok.DisplayForm renders a display form:

  >>> browser.open("http://localhost/manfred/@@display")
  >>> print browser.contents
  <html>...
  ...Manfred the Mammoth...
  ...Really big...
  ...

"""
import grok
from zope import schema
from zope.interface import Interface, implements
from zope.schema.fieldproperty import FieldProperty

class IMammoth(Interface):
    name = schema.TextLine(title=u"Name")
    size = schema.TextLine(title=u"Size", default=u"Quite normal")

class Mammoth(grok.Model):
    implements(IMammoth)
    
    name = FieldProperty(IMammoth['name'])    
    size = FieldProperty(IMammoth['size'])    

class Edit(grok.EditForm):
    pass

class Display(grok.DisplayForm):
    pass




"""
A template directory may only contain recognized template files:
  >>> from zope.deprecation.tests import warn
  >>> import warnings
  >>> saved_warn = warnings.warn
  >>> warnings.warn = warn


  >>> grok.testing.grok(__name__)
    From tests.py's showwarning():
    ... UserWarning: File 'invalid.txt' has an unrecognized extension in directory '...dirtemplatesonly_templates'...
    
  >>> warnings.warn = saved_warn

"""
import grok

class Mammoth(grok.Model):
    pass

class Index(grok.View):
    pass



##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""The Grok's Friendly Doctor.

Ask DocGrok and he will try his best, to keep you well informed about
everything hanging around in your Zope3 and your Grok Application.

See file `docgrok.txt` in this package to learn more about docgrok.

"""

import os
import sys # for sys.path
import types
import grok
import inspect
from urlparse import urlparse, urlunparse

import zope.component
from zope.app.folder.interfaces import IRootFolder
from zope.dottedname.resolve import resolve
from zope.interface.interface import InterfaceClass
from zope.security.proxy import isinstance
from zope.security.proxy import removeSecurityProxy
from zope.proxy import removeAllProxies

from zope.app.apidoc.codemodule.module import Module
from zope.app.apidoc.codemodule.class_ import Class
from zope.app.apidoc.codemodule.text import TextFile
from zope.app.apidoc.utilities import renderText
from zope.app.apidoc.utilities import getFunctionSignature
from zope.app.apidoc.utilities import getPythonPath, getPermissionIds
from zope.app.apidoc.utilities import isReferencable

import grok.interfaces
from grok.interfaces import IApplication
from martian.scan import is_package, ModuleInfo
from martian import ClassGrokker, ModuleGrokker
from grok.admin.objectinfo import ZopeObjectInfo

# This is the name under which the docgrok object-browser can be
# reached.
DOCGROK_ITEM_NAMESPACE = 'docgrok-obj'

grok.context(IRootFolder)

def find_filepath(dotted_path):
    """Find the filepath for a dotted name.

    If a dotted name denotes a filename we try to find its path
    by concatenating it with the system paths and looking for an
    existing file. Every dot in the filename thereby can be part
    of the filename or of its path. Therefore we check the
    several possible dirname/filename combinations possible.

    Returns None if no suitable filepath can be found.

    This functions does *not* look for Python elements like classes,
    interfaces and the files where they were defined. Use `resolve()`
    and the `__file__` attribute for examining this kind of stuff
    instead.

    This function finds the location of text files and the like, as
    far as they are placed inside some Python path.
    """
    currpath = dotted_path
    currname = ""
    while '.' in currpath:
        currpath, name = currpath.rsplit('.', 1)
        if currname != "":
            currname = "." + currname
        currname = name + currname
        tmp_path = ""
        for elem in currpath.split('.'):
            tmp_path = os.path.join(tmp_path, elem)
        for syspath in sys.path:
            filepath_to_check = os.path.join(syspath, tmp_path, currname)
            if os.path.isfile(filepath_to_check):
                return filepath_to_check
    return None

class DocGrokHandler(object):
    """A handler for DocGrok objects.

    The solely purpose of DocGrokHandlers is to determine, whether a
    given dotted path denotes a special type of object. If so, it
    should pass back an appropriate DocGrok object.

    In plain English: DocGrokHandlers find a doctor for any object.

    All we expect a DocGrokHandler to support, is a method
    ``getDoctor``, which takes a dotted path (and optional the object
    denoted by this path), to deliver an appropriate doctor or
    ``None``.
    """
    def getDoctor(self, dotted_path, obj=None):
        """The default docfinder cannot serve.
        """
        return


class DocGrokModuleHandler(DocGrokHandler):
    def getDoctor(self, dotted_path, obj=None):
        """Find a doctor for modules or None, if the dotted_path does
        not denote a module.
        """
        if obj is None:
            try:
                obj = resolve(dotted_path)
            except ImportError:
                return
        if not hasattr(obj, '__file__'):
            return
        if not is_package(os.path.dirname(obj.__file__)):
            return
        if os.path.basename(obj.__file__) in ['__init__.py',
                                             '__init__.pyc',
                                             '__init__.pyo']:
            return
        return DocGrokModule(dotted_path)


class DocGrokPackageHandler(DocGrokHandler):
    def getDoctor(self, dotted_path, obj=None):
        """Determine, whether the given path/obj references a Python
        package.
        """
        if obj is None:
            try:
                obj = resolve(dotted_path)
            except ImportError:
                return
        if not hasattr(obj, '__file__'):
            return
        if not is_package(os.path.dirname(obj.__file__)):
            return
        if os.path.basename(obj.__file__) not in ['__init__.py',
                                                 '__init__.pyc',
                                                 '__init__.pyo']:
            return
        return DocGrokPackage(dotted_path)


class DocGrokInterfaceHandler(DocGrokHandler):
    def getDoctor(self, dotted_path, obj=None):
        """Determine, whether the given path/obj references an interface.
        """
        if obj is None:
            try:
                obj = resolve(dotted_path)
            except ImportError:
                return
        if not isinstance(
            removeAllProxies(obj), InterfaceClass):
            return
        return DocGrokInterface(dotted_path)


class DocGrokClassHandler(DocGrokHandler):
    def getDoctor(self, dotted_path, obj=None):
        """Determine, whether the given path/obj references a Python class.
        """
        if obj is None:
            try:
                obj = resolve(dotted_path)
            except ImportError:
                return
        if not isinstance(obj, (types.ClassType, type)):
            return
        return DocGrokClass(dotted_path)


class DocGrokGrokApplicationHandler(DocGrokHandler):
    def getDoctor(self, dotted_path, obj=None):
        """Determine, whether the given path/obj references a Grok application.
        """
        if obj is None:
            try:
                obj = resolve(dotted_path)
            except ImportError:
                None
        try:
            if not IApplication.implementedBy(obj):
                return
        except TypeError:
            return
        return DocGrokGrokApplication(dotted_path)


class DocGrokTextFileHandler(DocGrokHandler):
    def getDoctor(self, dotted_path, obj=None):
        """Determine whether the dotted_path denotes a textfile.
        """
        if obj is not None:
            # Textfiles that are objects, are not text files.
            return
        if os.path.splitext(dotted_path)[1] != u'.txt':
            return
        return DocGrokTextFile(dotted_path)


def docgrok_handle(dotted_path):
    """Find a doctor specialized for certain things.
    """
    try:
        ob = resolve(dotted_path)
    except ImportError:
        # There is no object of that name. Give back 404.
        # TODO: Do something more intelligent, offer a search.
        if not find_filepath(dotted_path):
            return
        ob = None
    except:
        return

    for handler in docgrok_handlers:
        if type(handler) is not type({}):
            continue
        if 'docgrok_handler' not in handler.keys():
            continue
        spec_handler = handler['docgrok_handler']()
        if not isinstance(spec_handler, DocGrokHandler):
            continue
        doc_grok = spec_handler.getDoctor(dotted_path, ob)
        if doc_grok is None:
            continue
        return doc_grok
    # No special doctor could be found.
    return DocGrok(dotted_path)

def getInterfaceInfo(iface):
    if iface is None:
        return
    path = getPythonPath(iface)
    return {'path': path,
            'url': isReferencable(path) and path or None}


class DocGrokGrokker(ClassGrokker):
    """A grokker that groks DocGroks.

    This grokker can help to 'plugin' different docgroks in an easy
    way. You can register docgroks for your special classes, modules,
    things. All required, is a function, that determines the correct
    kind of thing, you like to offer a docgrok for and returns a
    specialized docgrok or None (in case the thing examined is not the
    kind of thing your docgrok is a specialist for).

    Unfortunately, order counts here. If one docgrok handler is able
    to deliver a specialized docgrok object, no further invesitgation
    will be done.

    In principle, the following should work. First we import the
    docgrok module, because it contains a more specific grokker: the
    InstanceGrokker 'docgrok_grokker' ::

      >>> from grok.admin import docgrok

    Then we create an (empty) 'ModuleGrokker'. 'ModuleGrokkers' can
    grok whole modules. ::

      >>> from martian import ModuleGrokker
      >>> module_grokker = ModuleGrokker()

    Then we register the 'docgrok_grokker', which should contain some
    base handlers for modules, classes, etc. by default::

      >>> module_grokker.register(docgrok.docgrok_handler_grokker)

    The 'docgrok_handler_grokker' is an instance of 'DocGrokGrokker'::

      >>> from grok.admin.docgrok import DocGrokGrokker
      >>> isinstance(docgrok.docgrok_handler_grokker, DocGrokGrokker)
      True

    That's it.

    """
    component_class = DocGrokHandler

    def grok(self, name, obj, **kw):
        if not issubclass(obj, DocGrokHandler):
            return
        if not hasattr(obj, 'getDoctor'):
            return
        docgrok_handlers.insert(0, {'name':name,
                                    'docgrok_handler':obj})
        return True


class DocGrok(grok.Model):
    """DocGrok helps us finding out things about ourselves.

    There are DocGroks for packages, modules, interfaces, etc., each
    one a specialist for a certain type of element. 'Pure' DocGroks
    build the root of this specialist hierarchy and care for objects,
    which can not be handled by other, more specialized DocGroks.

    DocGrok offers a minimum of information but can easily be extended in
    derived classes.
    """
    path = None
    _traversal_root = None

    def __init__(self, dotted_path):
        self.path = dotted_path

    def getPath(self):
        return self.path

    def getFilePath(self):
        try:
            ob = resolve(self.path)
            return hasattr(ob, __file__) and os.path.dirname(ob.__file__) or None
        except ImportError:
            pass
        return find_filepath(self.path)

    def getDoc(self, heading_only=False):
        """Get the doc string of the module STX formatted.
        """
        if hasattr(self, "apidoc") and hasattr(
            self.apidoc, "getDocString"):
            text = self.apidoc.getDocString()
        else:
            return
        if text is None:
            return
        lines = text.strip().split('\n')
        if len(lines) and heading_only:
            # Find first empty line to separate heading from trailing text.
            headlines = []
            for line in lines:
                if line.strip() == "":
                    break
                headlines.append(line)
            lines = headlines
        # Get rid of possible CVS id.
        lines = [line for line in lines if not line.startswith('$Id')]
        return renderText('\n'.join(lines), self.path)


    def traverse(self,patient):
        """ Do special traversing inside the surgery.

        Inside the docgrok-'namespace' we only accept DocGroks and
        colleagues. Each DocGrok cares for a patient represented by a
        path. This path might denote an object in the ZODB or in the
        python path.

        """
        if patient == "index.html":
            return self
        if self.path is None:
            newpath = patient
        else:
            newpath = '.'.join([self.path, patient])

        doctor = docgrok_handle(newpath)

        if doctor is None:
            # There is nothing of that name. Give back 404.
            # XXX Do something more intelligent, offer a search.
            return
        doctor.__parent__ = self
        doctor.__name__ = patient
        doctor._traversal_root = self._traversal_root
        doctor.path = newpath
        return doctor
    pass

def getItemLink(name, baseurl):
    """Get a link to a docgrok item out of an item name and a base URL.

    A docgrok item is any object, which is a 'subobject' of another
    object, for example an attribute, a memberfunction, an annotation
    or a sequence item (if the parent object is a sequence). Those
    objects are not neccessarily directly accessible, but need to be
    denoted for the object browser. We put 'docgrok-item' as marker at
    the beginning of the URL to enable the docgrok traverser and to
    handle those URLs in a special way. Such the objectbrowser can
    handle those 'unaccessible' items.
    """
    url = list(urlparse(baseurl))
    path = url[2]
    if path.startswith('/' + DOCGROK_ITEM_NAMESPACE + '/'):
        path = path[len(DOCGROK_ITEM_NAMESPACE) + 2:]
    if path.endswith('/@@inspect.html') or path.endswith('/inspect.html'):
        path = path.rsplit('/', 1)
    path = "/%s/%s%s/@@inspect.html" % (DOCGROK_ITEM_NAMESPACE, path, name)
    path = path.replace('//', '/')
    url[2] = path
    return urlunparse(url)


class DocGrokTraverser(grok.Traverser):
    """If first URL element is 'docgrok', handle over to DocGrok.

    This traverser binds to the RootFolder, which means, it is only
    asked, when the publisher looks for elements in the Zope root (or
    another IRootFolder). The further traversing is done by the Docs'
    own traverser in it's model. See method `traverse()` in DocGrok.
    """
    grok.context(IRootFolder)

    def traverse(self,path):
        if path == DOCGROK_ITEM_NAMESPACE:
            # The objectbrowser is called...
            obj_info = ZopeObjectInfo(self.context)
            obj_info.__parent__ = self.context
            obj_info.__name__ = DOCGROK_ITEM_NAMESPACE
            return obj_info

        if path == "docgrok":
            doctor = DocGrok(None)
            # Giving a __parent__ and a __name__, we make things
            # locatable in sense of ILocatable.
            doctor.__parent__ = self.context
            doctor.__name__ = 'docgrok'
            doctor._traversal_root = doctor
            return doctor
        return


class DocGrokPackage(DocGrok):
    """This doctor cares for python packages.
    """
    path=None
    apidoc = None
    _traversal_root = None

    def __init__(self,dotted_path):
        self.path = dotted_path
        self._module = resolve(self.path)
        # In apidoc packages are handled like modules...
        self.apidoc = Module(None, None, self._module, True)

    def getDocString(self):
        return self.apidoc.getDocString()

    def getFilePath(self):
        ob = resolve(self.path)
        return os.path.dirname(ob.__file__) + '/'

    def _getModuleInfos(self, filter_func=lambda x:x):
        """Get modules and packages of a package.

        The filter function will be applied to a list of modules and
        packages of type grok.scan.ModuleInfo.
        """
        ob = resolve(self.path)
        filename = ob.__file__
        module_info = ModuleInfo(filename, self.path)
        try:
            infos = module_info.getSubModuleInfos(exclude_tests=False)
        except TypeError:
            # Another version of martian.scan is installed
            infos = module_info.getSubModuleInfos()
        if filter_func is not None:
            infos = filter(filter_func, infos)
        result = []
        for info in infos:
            subresult = {}
            # Build a url string from dotted path...
            mod_path = "docgrok"
            for path_part in info.dotted_name.split('.'):
                mod_path = os.path.join(mod_path, path_part)
            subresult = {
                'url' : mod_path,
                'name' : info.name,
                'dotted_name' : info.dotted_name
                }
            result.append(subresult)
        return result


    def getModuleInfos(self):
        """Get the modules inside a package.
        """
        filter_func = lambda x: not x.isPackage()
        return self._getModuleInfos(filter_func)

    def getSubPackageInfos(self):
        """Get the subpackages inside a package.
        """
        filter_func = lambda x: x.isPackage()
        return self._getModuleInfos(filter_func)

    def getTextFiles(self):
        """Get the text files inside a package.
        """
        filter_func = lambda x: x.isinstance(TextFile)
        return self._getModuleInfos(filter_func)

    def getChildren(self):
        result = self.apidoc.items()
        result.sort(lambda x,y:cmp(x[0], y[0]))
        return result



class DocGrokModule(DocGrokPackage):
    """This doctor cares for python modules.
    """

    def getFilePath(self):
        ob = resolve(self.path)
        filename = ob.__file__
        if filename.endswith('o') or filename.endswith('c'):
            filename = filename[:-1]
        return filename


class DocGrokClass(DocGrokPackage):
    """This doctor cares for classes.
    """
    def __init__(self,dotted_path):
        self.path = dotted_path
        self.klass = resolve(self.path)
        self.module_path, self.name = dotted_path.rsplit('.',1)
        self.module = resolve(self.module_path)
        mod_apidoc = Module(None, None, self.module, False)
        self.apidoc = Class(mod_apidoc, self.name, self.klass)

    def getFilePath(self):
        if not hasattr(self.module, "__file__"):
            return
        filename = self.module.__file__
        if filename.endswith('o') or filename.endswith('c'):
            filename = filename[:-1]
        return filename

    def getAttributes(self):
        """Get the attributes of this class."""
        attrs = []
        # See remark in getMethods()
        klass = removeSecurityProxy(self.apidoc)
        for name, attr, iface in klass.getAttributes():
            entry = {'name': name,
                     'value': `attr`,
                     'type': type(attr).__name__,
                     'interface': iface
                }
            entry.update(getPermissionIds(name,klass.getSecurityChecker()))
            attrs.append(entry)
        return attrs

    def getMethods(self):
        """Get all methods of this class."""
        methods = []
        # remove the security proxy, so that `attr` is not proxied. We could
        # unproxy `attr` for each turn, but that would be less efficient.
        #
        # `getPermissionIds()` also expects the class's security checker not
        # to be proxied.
        klass = removeSecurityProxy(self.apidoc)
        for name, attr, iface in klass.getMethodDescriptors():
            entry = {'name': name,
                     'signature': "(...)",
                     'doc':attr.__doc__ or '',
                     'attr' : attr,
                     'interface' : iface}
            entry.update(getPermissionIds(name, klass.getSecurityChecker()))
            methods.append(entry)

        for name, attr, iface in klass.getMethods():
            entry = {'name': name,
                     'signature': getFunctionSignature(attr),
                     'doc':attr.__doc__ or '',
                     'attr' : attr,
                     'interface' : iface}
            entry.update(getPermissionIds(name, klass.getSecurityChecker()))
            methods.append(entry)
        return methods


class DocGrokInterface(DocGrokClass):
    """This doctor cares for interfaces.
    """
    def __init__(self,dotted_path):
        self.path = dotted_path
        self.klass = resolve(self.path)
        self.module_path, self.name = dotted_path.rsplit('.',1)
        self.module = resolve(self.module_path)
        mod_apidoc = Module(None, None, self.module, False)
        self.apidoc = Class(mod_apidoc, self.name, self.klass)

    def getFilePath(self):
        if not hasattr(self.module, "__file__"):
            return
        filename = self.module.__file__
        if filename.endswith('o') or filename.endswith('c'):
            filename = filename[:-1]
        return filename

class DocGrokGrokApplication(DocGrokClass):
    """This doctor cares for Grok applications and components.
    """
    pass

class DocGrokTextFile(DocGrok):
    """This doctor cares for text files.
    """

    def __init__(self,dotted_path):
        self.path = dotted_path
        self.filepath = find_filepath(self.path)
        self.filename = os.path.basename(self.filepath)


    def getPackagePath(self):
        """Return package path as dotted name.
        """
        dot_num_in_filename = len([x for x in self.filename if x == '.'])
        parts = self.path.rsplit('.', dot_num_in_filename + 1)
        return parts[0]

    def getContent(self):
        """Get file content UTF-8 encoded.
        """
        file = open(self.filepath, 'rU')
        content = file.read()
        file.close()
        return content.decode('utf-8')

# The docgroks registry.
#
# We register 'manually', because the handlers
# are defined in the same module.
docgrok_handlers = []

docgrok_handler_grokker = DocGrokGrokker()
docgrok_handler_grokker.grok('module', DocGrokModuleHandler)
docgrok_handler_grokker.grok('package', DocGrokPackageHandler)
docgrok_handler_grokker.grok('interface', DocGrokInterfaceHandler)
docgrok_handler_grokker.grok('class', DocGrokClassHandler)
docgrok_handler_grokker.grok('grokapplication',
                             DocGrokGrokApplicationHandler)
docgrok_handler_grokker.grok('textfile', DocGrokTextFileHandler)
docgrok_handlers_grokker = ModuleGrokker()
docgrok_handlers_grokker.register(docgrok_handler_grokker)




import grok

class Mammoth(grok.Model):
    pass

class Index(grok.View):
    pass

index = grok.PageTemplate("""\
<html>
<body>
<a tal:attributes="href static/file.txt">Some text in a file</a>
</body>
</html>""")



# -*- coding: latin-1 -*-
"""
You can only pass unicode to `grok.name`:

  >>> pass_unicode()
  >>> pass_encodedstring()
  Traceback (most recent call last):
    ...
  GrokImportError: You can only pass unicode or ASCII to grok.name.
  >>> pass_object()
  Traceback (most recent call last):
    ...
  GrokImportError: You can only pass unicode or ASCII to grok.name.

"""
import grok

def pass_unicode():
    class View(object):
        grok.name(u'name')

def pass_encodedstring():
    class View(object):
        grok.name("ölkj")

def pass_object():
    class View(object):
        grok.name(object())



import types
from zope import interface, event, lifecycleevent
from zope.interface.interfaces import IInterface
from zope.formlib import form
from zope.schema.interfaces import IField

class action(form.action):
    """We override the action decorator we pass in our custom Action.
    """
    def __call__(self, success):
        action = Action(self.label, success=success, **self.options)
        self.actions.append(action)
        return action

class Action(form.Action):
    def success(self, data):
        if self.success_handler is not None:
            return self.success_handler(self.form, **data)

def Fields(*args, **kw):
    fields = []
    for key, value in kw.items():
        if IField.providedBy(value):
            value.__name__ = key
            fields.append(value)
            del kw[key]
    fields.sort(key=lambda field: field.order)
    return form.Fields(*(args + tuple(fields)), **kw)

def get_auto_fields(context):
    """Get the form fields for context.
    """
    # for an interface context, we generate them from that interface
    if IInterface.providedBy(context):
        return form.Fields(context)
    # if we have a non-interface context, we're autogenerating them
    # from any schemas defined by the context
    fields = form.Fields(*most_specialized_interfaces(context))
    # we pull in this field by default, but we don't want it in our form
    fields = fields.omit('__name__')
    return fields

AutoFields = get_auto_fields

def most_specialized_interfaces(context):
    """Get interfaces for an object without any duplicates.

    Interfaces in a declaration for an object may already have been seen
    because it is also inherited by another interface. Don't return the
    interface twice, as that would result in duplicate names when creating
    the form.
    """
    declaration = interface.implementedBy(context)
    seen = []
    for iface in declaration.flattened():
        if interface_seen(seen, iface):
            continue
        seen.append(iface)
    return seen

def interface_seen(seen, iface):
    """Return True if interface already is seen.
    """
    for seen_iface in seen:
        if seen_iface.extends(iface):
            return True
    return False

def apply_data(context, form_fields, data, adapters=None, update=False):
    """Save form data (``data`` dict) on a ``context`` object.

    This is a beefed up version of zope.formlib.form.applyChanges().
    It allows you to specify whether values should be compared with
    the attributes on already existing objects or not, using the
    ``update`` parameter.

    Unlike zope.formlib.form.applyChanges(), it will return a
    dictionary of interfaces and their fields that were changed.  This
    is necessary to appropriately send IObjectModifiedEvents.
    """
    if adapters is None:
        adapters = {}

    changes = {}

    for form_field in form_fields:
        field = form_field.field
        # Adapt context, if necessary
        interface = form_field.interface
        adapter = adapters.get(interface)
        if adapter is None:
            if interface is None:
                adapter = context
            else:
                adapter = interface(context)
            adapters[interface] = adapter

        name = form_field.__name__
        newvalue = data.get(name, form_field) # using form_field as marker

        if update:
            if ((newvalue is not form_field) and
                (field.get(adapter) != newvalue)):
                field.set(adapter, newvalue)
                changes.setdefault(interface, []).append(name)
        else:
            if newvalue is not form_field:
                field.set(adapter, newvalue)
                changes.setdefault(interface, []).append(name)

    return changes

def apply_data_event(context, form_fields, data, adapters=None, update=False):
    """Like apply_data, but also sends an IObjectModifiedEvent.
    """
    changes = apply_data(context, form_fields, data, adapters, update)

    if changes:
        descriptions = []
        for interface, names in changes.items():
            descriptions.append(lifecycleevent.Attributes(interface, *names))
        event.notify(lifecycleevent.ObjectModifiedEvent(context, *descriptions))

    return changes



# -*- coding: UTF-8 -*-
"""
Views have a method that can be used to construct URLs:

  >>> herd = Herd()
  >>> getRootFolder()['herd'] = herd
  >>> manfred = Mammoth()
  >>> herd['manfred'] = manfred

The views in this test implement self.url():

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/herd/manfred/index")
  >>> print browser.contents
  http://localhost/herd/manfred/index
  >>> browser.open("http://localhost/herd/manfred/another")
  >>> print browser.contents
  http://localhost/herd/manfred/another
  >>> browser.open("http://localhost/herd/manfred/yetanother")
  >>> print browser.contents
  http://localhost/herd/manfred/yetanother
  
We get the views manually so we can do a greater variety of url() calls:

  >>> from zope import component
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> index_view = component.getMultiAdapter((manfred, request), name='index')
  >>> index_view.url()
  'http://127.0.0.1/herd/manfred/index'
  >>> another_view = component.getMultiAdapter((manfred, request),
  ...                                              name='another')
  >>> another_view.url()
  'http://127.0.0.1/herd/manfred/another'
  >>> yet_another_view = component.getMultiAdapter((manfred, request),
  ...                                              name='yetanother')
  >>> yet_another_view.url()
  'http://127.0.0.1/herd/manfred/yetanother'

Now let's get a URL for a specific object:

  >>> index_view.url(manfred)
  'http://127.0.0.1/herd/manfred'

This works with any other view too (as they share the same request):

  >>> another_view.url(manfred)
  'http://127.0.0.1/herd/manfred'

This shows that the default argument is the view itself:

  >>> another_view.url(another_view)
  'http://127.0.0.1/herd/manfred/another'

We can get the URL for any object in content-space:

  >>> another_view.url(herd)
  'http://127.0.0.1/herd'

We can also pass a name along with this, to generate a URL to a
particular view on the object:

  >>> another_view.url(herd, 'something')
  'http://127.0.0.1/herd/something'

It works properly in the face of non-ascii characters in URLs:

  >>> url = another_view.url(herd, unicode('árgh', 'UTF-8'))
  >>> url
  'http://127.0.0.1/herd/%C3%A1rgh'
  >>> import urllib
  >>> expected = unicode('http://127.0.0.1/herd/árgh', 'UTF-8')
  >>> urllib.unquote(url).decode('utf-8') == expected
  True

It's also possible to just pass in a name. In this case, a URL to that
view on the context object will be constructed:

  >>> another_view.url('yet_another_view')
  'http://127.0.0.1/herd/manfred/yet_another_view'

Some combinations of arguments just don't make sense:

  >>> another_view.url('foo', 'bar')
  Traceback (most recent call last):
    ...
  TypeError: url() takes either obj argument, obj, string arguments, or
  string argument
  >>> another_view.url('foo', herd)
  Traceback (most recent call last):
    ...
  TypeError: url() takes either obj argument, obj, string arguments, or
  string argument
  >>> another_view.url(herd, 'bar', 'baz')
  Traceback (most recent call last):
    ...
  TypeError: url() takes at most 3 arguments (4 given)
  
"""
import grok

class Herd(grok.Container, grok.Model):
    pass

class Mammoth(grok.Model):
    pass

grok.context(Mammoth)

class Index(grok.View):
    def render(self):
        return self.url()
    
class Another(grok.View):
    def render(self):
        return self.url()

class YetAnother(grok.View):
    pass

yetanother = grok.PageTemplate('<p tal:replace="view/url" />')



# -*- coding: latin-1 -*-
# Copyright (c) 2007 gocept gmbh & co. kg
# See also LICENSE.txt
# $Id$
"""Support for flash-messages in the grok admin UI."""

import zope.interface
import zope.component

import grok

import z3c.flashmessage.interfaces
import z3c.flashmessage.sources
import z3c.flashmessage.receiver


class Messages(grok.View):

    grok.context(zope.interface.Interface)

    @property
    def messages(self):
        receiver = zope.component.getUtility(
            z3c.flashmessage.interfaces.IMessageReceiver)
        return receiver.receive()



grok.global_utility(z3c.flashmessage.sources.SessionMessageSource,
                    name='session')
grok.global_utility(z3c.flashmessage.receiver.GlobalMessageReceiver)



"""
If multiple templates can be found, one in the module and one in the
template directory, there is an error:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  ConfigurationExecutionError: martian.error.GrokError: Conflicting templates found for name 'cavepainting' in module
  <module 'grok.tests.view.dirandinlinetemplate' from ...

"""
import grok

class Mammoth(grok.Model):
    pass

class CavePainting(grok.View):
    pass

cavepainting = grok.PageTemplate("nothing")



"""
It is an error for the 'static' directory to be a python package:

  >>> import grok
  >>> grok.testing.grok('grok.tests.static.staticispackage_fixture')
  Traceback (most recent call last):
    ...
  GrokError: The 'static' resource directory must not be a python package.
"""



"""
Using the @grok.action decorator, different actions can be defined on
a grok.Form. When @grok.action is used, the default behaviour (the
'Apply' action) is not available anymore, but it can triggered
manually by calling self.applyData(object, data).

  >>> getRootFolder()["manfred"] = Mammoth()

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/manfred/@@edit")
  >>> browser.getControl(name="form.name").value = "Manfred the Mammoth"
  >>> browser.getControl(name="form.size").value = "Really big"
  >>> browser.getControl("Apply").click()
  >>> print browser.contents
  <html>...
  ...Modified!...
  ...Manfred the Mammoth...
  ...Really big...
  ...

Save again without any changes:

  >>> browser.getControl("Apply").click()
  >>> print browser.contents
  <html>...
  ...No changes!...
  ...

  >>> browser.open("http://localhost/manfred/@@edit")
  >>> browser.getControl(name="form.name").value = "Manfred the Second"
  >>> browser.getControl("Hairy").click()
  >>> print browser.contents
  <html>...
  ...Manfred the Second...
  ...Really big and hairy...
  ...

  >>> browser.open("http://localhost/manfred/meet")
  >>> browser.getControl(name="form.other").value = "Ellie"
  >>> browser.getControl("Meet").click()
  >>> print browser.contents
  Manfred the Second meets Ellie
"""
import grok
from zope import schema
from zope.interface import Interface, implements
from zope.schema.fieldproperty import FieldProperty

class IMammoth(Interface):
    name = schema.TextLine(title=u"Name")
    size = schema.TextLine(title=u"Size", default=u"Quite normal")

class Mammoth(grok.Model):
    implements(IMammoth)
    
    name = FieldProperty(IMammoth['name'])    
    size = FieldProperty(IMammoth['size'])    

class Edit(grok.EditForm):
    @grok.action("Apply")
    def handle_apply(self, **data):
        if self.applyData(self.context, **data):
            self.status = 'Modified!'
        else:
            self.status = 'No changes!'

    @grok.action("Hairy")
    def handle_hairy(self, **data):
        self.applyData(self.context, **data)
        self.context.size += " and hairy"

class Meet(grok.Form):
    form_fields = grok.Fields(
        other = schema.TextLine(title=u'Mammoth to meet with')
        )

    @grok.action('Meet')
    def meet(self, other):
        return self.context.name + ' meets ' + other



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok ZCML directives."""

from zope.interface import Interface
from zope.configuration.fields import GlobalObject
from zope.configuration.config import ConfigurationMachine

import martian
from martian import scan
from martian.error import GrokError

class IGrokDirective(Interface):
    """Grok a package or module."""

    package = GlobalObject(
        title=u"Package",
        description=u"The package or module to be analyzed by grok.",
        required=False,
        )

# add a cleanup hook so that grok will bootstrap itself again whenever
# the Component Architecture is torn down.
def resetBootstrap():
    # we need to make sure that the grokker registry is clean again
    the_module_grokker.clear()
from zope.testing.cleanup import addCleanUp
addCleanUp(resetBootstrap)

the_multi_grokker = martian.MetaMultiGrokker()
the_module_grokker = martian.ModuleGrokker(the_multi_grokker)

def skip_tests(name):
    return name in ['tests', 'ftests']

def grokDirective(_context, package):
    do_grok(package.__name__, _context)

def do_grok(dotted_name, config):
    martian.grok_dotted_name(
        dotted_name, the_module_grokker, exclude_filter=skip_tests,
        config=config
        )



"""
  >>> methods = grok.util.public_methods_from_class(A)
  >>> sorted([m.__name__ for m in methods])
  ['should_also_be_public', 'should_be_public']

"""
import grok
import grok.util

class A(object):

    def __init__(self):
        pass # this method is ignored

    def __call__(self):
        pass # this method is ignored

    def __double_underscored(self):
        pass # this method is ignored

    def _single_underscored(self):
        pass # this method is ignored

    def should_be_public(self):
        pass # this method is found

    def should_also_be_public(self):
        pass # this method is found



"""

  >>> grok.testing.grok(__name__)

View with an associated PageTemplate that is referred to using
``grok.template``:

  >>> manfred = Mammoth()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  >>> view = component.getMultiAdapter((manfred, request), name='painting')
  >>> print view()
  <html><body><h1>GROK PAINT MAMMOTH!</h1></body></html>

``grok.name`` and ``grok.template`` can be combined:

  >>> view = component.getMultiAdapter((manfred, request), name='meal')
  >>> print view()
  <html><body><h1>GROK EAT MAMMOTH!</h1></body></html>

"""
import grok

class Mammoth(grok.Model):
    pass

class Painting(grok.View):
    grok.template('cavepainting')

cavepainting = grok.PageTemplate("""\
<html><body><h1>GROK PAINT MAMMOTH!</h1></body></html>
""")

class Food(grok.View):
    grok.template('food_template')
    grok.name('meal')

food_template = grok.PageTemplate("""\
<html><body><h1>GROK EAT MAMMOTH!</h1></body></html>
""")



"""
A grok.EditForm is a special view that renders an edit form.

  >>> from zope import component
  >>> from zope.publisher.browser import TestRequest
  >>> grok.testing.grok(__name__)
  >>> manfred = Mammoth()

  >>> request = TestRequest()
  >>> view = component.getMultiAdapter((manfred, request), name='edit')
  >>> len(view.form_fields)
  2
  >>> [w.__name__ for w in view.form_fields]
  ['name', 'size']

Let's assert that forms are indeed Zope 3-style browser views and
browser pages:

  >>> from zope.publisher.interfaces.browser import IBrowserPage, IBrowserView
  >>> IBrowserPage.providedBy(view)
  True
  >>> IBrowserView.providedBy(view)
  True

It is important to keep the order of the fields:

  >>> view = component.getMultiAdapter(
  ...    (DifferentMammoth(), request), name='editdifferent')
  >>> len(view.form_fields)
  2
  >>> [w.__name__ for w in view.form_fields]
  ['size', 'name']

"""
import grok
from zope import schema
from zope.interface import Interface, implements

class IMammoth(Interface):
    name = schema.TextLine(title=u"Name")
    size = schema.TextLine(title=u"Size", default=u"Quite normal")

class Mammoth(grok.Model):
    implements(IMammoth)

class Edit(grok.EditForm):
    grok.context(Mammoth)

class IDifferentMammoth(Interface):
    # mind the different order of fields
    size = schema.TextLine(title=u"Size", default=u"Quite normal")
    name = schema.TextLine(title=u"Name")

class DifferentMammoth(grok.Model):
    implements(IDifferentMammoth)

class EditDifferent(grok.EditForm):
    grok.context(DifferentMammoth)




"""
A Grokker can declare that scanning should continue, so that other Grokkers can
still perform actions on the grokked components.

Here we define AlphaGrokker which has higher priority than BetaGrokker but does
not block BetaGrokker from picking up the same component::

    >>> grok.testing.grok(__name__)

In the fixture there is AlphaBetaSub that inherits from both Alpha and Beta.
Thus, both Grokkers are executed, with AlphaGrokker coming before BetaGrokker::

    >>> grok.testing.grok('grok.tests.grokker.continue_scanning_fixture')
    alpha
    beta

"""
import grok


class Alpha(object):
    pass

class Beta(object):
    pass

class AlphaGrokker(grok.ClassGrokker):
    component_class = Alpha
    priority = 1 # we need to go before BetaGrokker

    def grok(self, name, factory, module_info, config, **kw):
        print "alpha"
        return True

class BetaGrokker(grok.ClassGrokker):
    component_class = Beta

    def grok(self, name, factory, module_info, config, **kw):
        print "beta"
        return True
    



"""
Templates can be specified in the same module as the view,
using a variable named `viewname_pt`:

  >>> grok.testing.grok(__name__)
  
  >>> manfred = Mammoth()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  >>> view = component.getMultiAdapter((manfred, request), name='cavepainting')
  >>> print view()
  <html>
  <body>
  <h1>Mammoth Cave Painting</h1>
  <ul>
    <li><zope.publisher.browser.TestRequest instance URL=http://127.0.0.1></li>
    <li><grok.tests.view.inline.CavePainting object at 0x...></li>
    <li><grok.tests.view.inline.Mammoth object at 0x...></li>
    <li><zope.app.pagetemplate.engine.TraversableModuleImporter object at 0x...></li>
  </ul>
  </body>
  </html>

Note that the CavePainting instance is bound to the ``view`` name in
the template.  This shows that the association of inline PageTemplate
and the view class is successful.

Finding a template does not depend on the view name, but on the class
name:

  >>> view = component.getMultiAdapter((manfred, request), name='hunting')
  >>> print view()
  <html><body><h1>GROK HUNT MAMMOTH!</h1></body></html>

"""
import grok

class Mammoth(grok.Model):
    pass

class CavePainting(grok.View):
    pass

cavepainting = grok.PageTemplate("""\
<html>
<body>
<h1 tal:content="string:Mammoth Cave Painting"/>
<ul>
  <li tal:content="structure python:repr(request)" />
  <li tal:content="structure nocall:view" />
  <li tal:content="structure nocall:context" />
  <li tal:content="structure nocall:modules" />
</ul>
</body>
</html>
""")

class Hunt(grok.View):
    grok.name('hunting')

hunt = grok.PageTemplate("""\
<html><body><h1>GROK HUNT MAMMOTH!</h1></body></html>
""")




"""
Filesystem-based templates, once grokked, can be changed.  The change
will automatically be picked up, reloading Zope is not necessary.

  >>> grok.testing.grok(__name__)
  >>> from zope.component import getMultiAdapter
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> view = getMultiAdapter((Mammoth(), request), name='index')
  >>> print view()
  before

Now we change the file:

  >>> import os.path
  >>> here = os.path.dirname(__file__)
  >>> template_file = os.path.join(here, 'templatereload_templates', 'index.pt')
  >>> template = open(template_file, 'w')
  >>> template.write('after')
  >>> template.close()

and find that the output of the view has changed as well:

  >>> print view()
  after

At last, we should change everything back to normal:

  >>> template = open(template_file, 'w')
  >>> template.write('before')
  >>> template.close()
"""
import grok

class Mammoth(grok.Model):
    pass

class Index(grok.View):
    pass



"""
It is too confusing to have a template that would be implicitly
associated with a view while that view already refers to another
template using grok.template.  Therefore there is an error:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  ConfigurationExecutionError: martian.error.GrokError: Multiple possible templates for view
  <class 'grok.tests.view.explicitimplicittemplate.Painting'>.
  It uses grok.template('cavepainting'), but there is also a template
  called 'painting'.
  in:
  
"""
import grok

class Mammoth(grok.Model):
    pass

class Painting(grok.View):
    grok.template('cavepainting')

cavepainting = grok.PageTemplate("GROK CAVEPAINT MAMMOTH!")
painting = grok.PageTemplate("GROK PAINT MAMMOTH!")



"""
Views have a redirect() method to easily create redirects:

  >>> getRootFolder()['manfred'] = manfred = Mammoth()

Since the index view redirects to mammoth, we expect to see the URL
point to mammoth:

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open('http://localhost/manfred')
  >>> browser.url
  'http://localhost/manfred/another'
  
"""
import grok

class Mammoth(grok.Model):
    pass

class Index(grok.View):
    def render(self):
        self.redirect(self.url('another'))

class Another(grok.View):
    def render(self):
        return "Another view"
    




"""
Views without a context cannot be grokked:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: No module-level context for
  <class 'grok.tests.view.missingcontext.Club'>, please use grok.context.

"""

import grok

class Club(grok.View):
    pass



"""
A template can be used by multiple views at the same time:

  >>> grok.testing.grok(__name__)

  >>> manfred = Mammoth()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component

  >>> view = component.getMultiAdapter((manfred, request), name='a')
  >>> print view()
  View A

  >>> view = component.getMultiAdapter((manfred, request), name='b')
  >>> print view()
  View A

It also works if templates are both associated explicitly:

  >>> view = component.getMultiAdapter((manfred, request), name='c')
  >>> print view()
  Template

  >>> view = component.getMultiAdapter((manfred, request), name='d')
  >>> print view()
  Template

Because the template is associated, we do not expect it to be
registered as its own view:

  >>> view = component.getMultiAdapter((manfred, request), name='templ')
  Traceback (most recent call last):
    ...
  ComponentLookupError:
  ((<grok.tests.view.twoviewsusetemplate.Mammoth object at 0x...>,
  <zope.publisher.browser.TestRequest instance URL=http://127.0.0.1>),
  <InterfaceClass zope.interface.Interface>, 'templ')


"""
import grok

class Mammoth(grok.Model):
    pass

class A(grok.View):
    pass

a = grok.PageTemplate("View A")

class B(grok.View):
    grok.template('a')

class C(grok.View):
    grok.template('templ')

class D(grok.View):
    grok.template('templ')

templ = grok.PageTemplate('Template')



##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Views for the grok admin UI"""

import grok
import os
import inspect
from urllib import urlencode

from grok.admin import docgrok
from grok.admin.docgrok import DocGrok, DocGrokPackage, DocGrokModule
from grok.admin.docgrok import DocGrokTextFile, DocGrokGrokApplication
from grok.admin.docgrok import DocGrokClass, DocGrokInterface, getItemLink

from grok.admin.objectinfo import ZopeObjectInfo
from grok.admin.utilities import getPathLinksForObject, getPathLinksForClass
from grok.admin.utilities import getPathLinksForDottedName, getParentURL

from ZODB.broken import Broken
from BTrees.OOBTree import OOBTree

import zope.component
from zope.interface import Interface
from zope.interface.interface import InterfaceClass
from zope.app.applicationcontrol.interfaces import IServerControl
from zope.app.applicationcontrol.applicationcontrol import applicationController
from zope.app.applicationcontrol.runtimeinfo import RuntimeInfo
from zope.app.applicationcontrol.browser.runtimeinfo import RuntimeInfoView
from zope.app.apidoc import utilities, codemodule
from zope.app.apidoc.utilities import getPythonPath, renderText, columnize
from zope.app.apidoc.codemodule.module import Module
from zope.app.apidoc.codemodule.class_ import Class
from zope.app.apidoc.codemodule.function import Function
from zope.app.apidoc.codemodule.text import TextFile
from zope.app.apidoc.codemodule.zcml import ZCMLFile
from zope.app.folder.interfaces import IRootFolder
from zope.app.security.interfaces import ILogout, IAuthentication
from zope.app.security.interfaces import IUnauthenticatedPrincipal
from zope.exceptions import DuplicationError
from zope.proxy import removeAllProxies
from zope.tal.taldefs import attrEscape

import z3c.flashmessage.interfaces


grok.context(IRootFolder)

class ManageApplications(grok.Permission):
    grok.name('grok.ManageApplications')

class Add(grok.View):
    """Add an application.
    """

    grok.require('grok.ManageApplications')

    def update(self, inspectapp=None, application=None):
        if inspectapp is not None:
            self.redirect(self.url("docgrok") + "/%s/index"%(application.replace('.','/'),))
        return

    def render(self, application, name, inspectapp=None):
        if name is None or name == "":
            self.redirect(self.url(self.context))
            return
        if name is None or name == "":
            self.redirect(self.url(self.context))
            return
        app = zope.component.getUtility(grok.interfaces.IApplication,
                                        name=application)
        try:
            self.context[name] = app()
            self.flash(u'Added %s `%s`.' % (application, name))
        except DuplicationError:
            self.flash(
                u'Name `%s` already in use. Please choose another name.' % (
                name,))
        self.redirect(self.url(self.context))


class Delete(grok.View):
    """Delete an application.
    """

    grok.require('grok.ManageApplications')

    def render(self, items=None):
        if items is None:
            self.redirect(self.url(self.context))
            return
        msg = u''
        if not isinstance(items, list):
            items = [items]
        for name in items:
            try:
                del self.context[name]
                msg = (u'%sApplication `%s` was successfully '
                       u'deleted.\n' % (msg, name))
            except AttributeError:
                # Object is broken.. Try it the hard way...
                # TODO: Try to repair before deleting.
                obj = self.context[name]
                if not hasattr(self.context, 'data'):
                    msg = (
                        u'%sCould not delete application `%s`: no '
                        u'`data` attribute found.\n' % (msg, name))
                    continue
                if not isinstance(self.context.data, OOBTree):
                    msg = (
                        u'%sCould not delete application `%s`: no '
                        u'`data` is not a BTree.\n' % (msg, name))
                    continue
                self.context.data.pop(name)
                self.context.data._p_changed = True
                msg = (u'%sBroken application `%s` was successfully '
                       u'deleted.\n' % (msg, name))

        self.flash(msg)
        self.redirect(self.url(self.context))


class GAIAView(grok.View):
    """A grok.View with a special application_url.

    We have to compute the application_url different from common
    grok.Views, because we have no root application object in the
    adminUI. To avoid mismatch, we also call it 'root_url'.

    """

    def root_url(self, name=None):
        obj = self.context
        result = ""
        while obj is not None:
            if __grok_context__.providedBy(obj):
                return self.url(obj, name)
            obj = obj.__parent__
        raise ValueError("No application nor root element found.")

    def in_docgrok(self):
        return '/docgrok/' in self.url() or 'inspect.html' in self.url()

    def is_authenticated(self):
        """Check, wether we are authenticated.
        """
        return not IUnauthenticatedPrincipal.providedBy(self.request.principal)


class GrokAdminMacros(GAIAView):
    """Provides the o-wrap layout."""

    grok.context(Interface)



class Inspect(GAIAView):
    """Basic object browser.
    """

    grok.context(Interface)
    grok.name(u'inspect.html')
    grok.require('grok.ManageApplications')

    _metadata = None

    def update(self, show_private=False, *args, **kw):
        obj = self.context
        if isinstance(self.context, ZopeObjectInfo):
            # When the docgrok-object traverser delivers content, then
            # we get a wrapped context: the meant object is wrapped
            # into a ZopeObjectInfo.
            obj = self.context.obj

        self.ob_info = ZopeObjectInfo(obj)
        ob_info = self.ob_info
        self.show_private = show_private
        root_url = self.root_url()
        parent = ob_info.getParent()
        parent = {'class_link':
                      parent and getPathLinksForObject(parent) or '',
                  'obj_link' : getItemLink('',getParentURL(self.url(''))),
                  'obj' : parent
                  }
        bases = [getPathLinksForClass(x) for x in ob_info.getBases()]
        bases.sort()

        ifaces = [getPathLinksForClass(x) for x in
                  ob_info.getProvidedInterfaces()]
        ifaces.sort()

        methods = [x for x in list(ob_info.getMethods())
                   if self.show_private or not x['name'].startswith('_')]
        for method in methods:
            if method['interface']:
                method['interface'] = getPathLinksForDottedName(
                    method['interface'], root_url)
            if method['doc']:
                method['doc'] = renderText(method['doc'], getattr(obj,'__module__', None))

        attrs = [x for x in list(ob_info.getAttributes())
                 if self.show_private or not x['name'].startswith('_')
                 ]
        for attr in attrs:
            if '.' in str(attr['type']):
                attr['type'] = getPathLinksForClass(attr['type'], root_url)
            else:
                attr['type'] = attrEscape(str(attr['type']))
            if attr['interface']:
                attr['interface'] = getPathLinksForDottedName(
                    attr['interface'], root_url)
            attr['obj'] = getattr(obj, attr['name'], None)
            attr['docgrok_link'] = getItemLink(attr['name'], self.url(''))
        attrs.sort(lambda x,y: x['name']>y['name'])

        seqitems = ob_info.getSequenceItems() or []
        for item in seqitems:
            if '.' in str(item['value_type']):
                item['value_type'] = getPathLinksForClass(item['value_type'],
                                                          root_url)
            else:
                item['value_type'] = attrEscape(str(item['value_type']))
            item['obj'] = obj[item['index']]
            item['docgrok_link'] = getItemLink(item['index'], self.url(''))
        seqitems.sort()

        mapitems = [x for x in ob_info.getMappingItems()
                    if self.show_private or not x['key'].startswith('_')]
        for item in mapitems:
            if '.' in str(item['value_type']):
                item['value_type'] = getPathLinksForClass(item['value_type'],
                                                          root_url)
            else:
                item['value_type'] = attrEscape(str(item['value_type']))
            item['obj'] = obj[item['key']]
            item['docgrok_link'] = getItemLink(item['key'], self.url(''))
        mapitems.sort(lambda x,y: x['key']>y['key'])

        annotations = [x for x in ob_info.getAnnotationsInfo()
                    if self.show_private or not x['key'].startswith('_')]
        for item in annotations:
            if '.' in str(item['value_type']):
                item['value_type'] = getPathLinksForClass(item['value_type'],
                                                          root_url)
            else:
                item['value_type'] = attrEscape(str(item['value_type']))
            item['docgrok_link'] = getItemLink(item['key'], self.url(''))
        annotations.sort(lambda x,y: x['key']>y['key'])


        self.info = {
            'name' : ob_info.getId() or u'<unnamed object>',
            'type' : getPathLinksForClass((getattr(obj,
                                                   '__class__',
                                                   None)
                                           or type(obj)), root_url),
            'obj_link' : getPathLinksForObject(obj, root_url),
            'moduleinfo' : ob_info.getmoduleinfo(),
            'modulename' : ob_info.getmodulename(),
            'ismodule' : ob_info.ismodule(),
            'isclass' : ob_info.isclass(),
            'ismethod' : ob_info.ismethod(),
            'isfunction' : ob_info.isfunction(),
            'iscode' : ob_info.iscode(),
            'isbuiltin' : ob_info.isbuiltin(),
            'isroutine' : ob_info.isroutine(),
            'issequence' : ob_info.isSequence(),
            'ismapping' : ob_info.isMapping(),
            'isannotatable' : ob_info.isAnnotatable(),
            'doc' : renderText(ob_info.getdoc(),None),
            'comments' : ob_info.getcomments(),
            'module' : ob_info.getmodule(),
            'sourcefile' : ob_info.getsourcefile(),
            'source' : ob_info.getsource(),
            'parent' : parent,
            'dotted_path' : ob_info.getPythonPath(),
            'provided_interfaces' : ob_info.getDirectlyProvidedInterfaces(),
            'interfaces' : ifaces,
            'bases' : bases,
            'attributes' : attrs,
            'methods' : methods,
            'sequenceitems' : seqitems,
            'mappingitems' : mapitems,
            'annotations' : annotations
            }


class Index(GAIAView):
    """A redirector to the real frontpage."""

    grok.name('index.html') # The root folder is not a grok.Model
    grok.require('grok.ManageApplications')

    def update(self):
        apps = zope.component.getAllUtilitiesRegisteredFor(
            grok.interfaces.IApplication)
        self.applications = ("%s.%s" % (x.__module__, x.__name__)
                             for x in apps)
        # Go to the first page immediately.
        self.redirect(self.url('applications'))


class Applications(GAIAView):
    """View for application management.

    """

    grok.name('applications')
    grok.require('grok.ManageApplications')

    def getDocOfApp(self, apppath, headonly = True):
        doctor = docgrok.docgrok_handle(apppath)
        result = doctor.getDoc(headonly)
        if result is None:
            result = ""
        return result

    def update(self):
        from ZODB import broken

        from zope.app.broken.broken import IBroken

        # Available apps...
        apps = zope.component.getAllUtilitiesRegisteredFor(
            grok.interfaces.IApplication)
        self.applications = (
            {'name': "%s.%s" % (x.__module__, x.__name__),
             'docurl':("%s.%s" % (x.__module__, x.__name__)).replace('.', '/')}
            for x in apps)

        # Installed apps...
        inst_apps = [x for x in self.context.values()
                     if hasattr(x, '__class__') and x.__class__ in apps
                     and not issubclass(x.__class__, Broken)]
        inst_apps.sort(lambda x, y: cmp(x.__name__, y.__name__))
        self.installed_applications = inst_apps

        # Broken apps...
        broken_apps = [{'obj':y, 'name':x} for x,y in self.context.items()
                       if isinstance(y, Broken)]
        broken_apps.sort(lambda x, y: cmp(x['name'], y['name']))
        self.broken_applications = broken_apps


class AdminMessageSource(grok.GlobalUtility):

    grok.name('admin')
    zope.interface.implements(z3c.flashmessage.interfaces.IMessageSource)

    message = None

    def send(self, message, type='admin'):
        self.message = z3c.flashmessage.message.PersistentMessage(message,
                                                                  type)

    def list(self, type=None):
        if self.message is None:
            return
        if type is None or self.message.type == type:
            yield self.message

    def delete(self, message):
        if message is self.message:
            self.message = None
        else:
            raise KeyError(message)


class Server(GAIAView):
    """Zope3 management screen."""

    grok.require('grok.ManageApplications')

    @property
    def server_control(self):
        return zope.component.getUtility(IServerControl)

    @property
    def runtime_info(self):
        riv = RuntimeInfoView()
        riv.context = applicationController
        return riv.runtimeInfo()

    @property
    def current_message(self):
        source = zope.component.getUtility(
          z3c.flashmessage.interfaces.IMessageSource, name='admin')
        messages = list(source.list())
        if messages:
            return messages[0]

    def update(self, time=None, restart=None, shutdown=None,
              admin_message=None, submitted=False):
        if not submitted:
            return
        # Admin message control
        source = zope.component.getUtility(
          z3c.flashmessage.interfaces.IMessageSource, name='admin')
        if admin_message is not None:
            source.send(admin_message)
        elif source.current_message:
            source.delete(source.current_message)

        # Restart control
        if time is not None:
            try:
                time = int(time)
            except:
                time = 0
        else:
            time = 0

        if restart is not None:
            self.server_control.restart(time)
        elif shutdown is not None:
            self.server_control.shutdown(time)

        self.redirect(self.url())


class Users(GAIAView):
    """Users management screen."""

    grok.name('users')
    grok.require('grok.ManageApplications')

    def getPrincipals(self):
        from grok.admin import AUTH_FOLDERNAME, USERFOLDER_NAME

        sm = self.context.getSiteManager()
        if AUTH_FOLDERNAME not in list(sm.keys()):
            return []
        pau = sm[AUTH_FOLDERNAME]
        if USERFOLDER_NAME not in list(pau.keys()):
            return []
        userfolder = pau[USERFOLDER_NAME]
        users = list(userfolder.search({'search':''}))
        return [userfolder.principalInfo(x) for x in users]


    def update(self):
        self.principals = self.getPrincipals()
        pass



def getDottedPathDict(dotted_path):
    """Get a dict containing parts of a dotted path as links.
    """
    if dotted_path is None:
        return {}

    result = []
    part_path = ""
    for part in dotted_path.split('.'):
        name = part
        if part_path != "":
            name = "." + part
        part_path += part
        result.append({
            'name':name,
            'url':"/docgrok/%s" % (part_path,)
            })
        part_path += "/"
    return result


class DocGrokView(GAIAView):
    """A base DocGrok view.

    This view is used for all things not covered by other, more
    specialized views.
    """

    grok.context(DocGrok)
    grok.name('index')
    grok.require('grok.ManageApplications')

    def getDoc(self, text=None, heading_only=False):
        """Get the doc string of the module STX formatted."""
        if text is None:
            return None
            if (hasattr(self.context, "apidoc") and
                hasattr(self.context.apidoc, "getDocString")):
                text = self.context.apidoc.getDocString()
            else:
                return None
        lines = text.strip().split('\n')
        if len(lines) and heading_only:
            # Find first empty line to separate heading from trailing text.
            headlines = []
            for line in lines:
                if line.strip() == "":
                    break
                headlines.append(line)
            lines = headlines
        # Get rid of possible CVS id.
        lines = [line for line in lines if not line.startswith('$Id')]
        return renderText('\n'.join(lines), self.context.getPath())

    def getDocHeading(self, text=None):
        return self.getDoc(text, True)

    def getPathParts(self, path=None):
        """Get parts of a dotted name as url and name parts.
        """
        if path is None:
            path = self.context.path
        if path is None:
            return None
        return getDottedPathDict(path)
        result = []
        part_path = ""
        for part in path.split('.'):
            name = part
            if part_path != "":
                name = "." + part
            part_path += part
            result.append({
                'name':name,
                'url':"/docgrok/%s" % (part_path,)
                })
            part_path += "/"
        return result

    def getEntries(self, columns=True):
        """Return info objects for all modules and classes in the
        associated apidoc container.

        """
        if (not hasattr(self.context, "apidoc") or
            not hasattr(self.context.apidoc, "items")):
            return None
        entries = []
        for name, obj in self.context.apidoc.items():
            entry = {
                'name': name,
                'obj' : obj,
                'path': getPythonPath(removeAllProxies(obj)),
                'url' : u'',
                'doc' : None,
                'ispackage' : False,
                'ismodule' : False,
                'isinterface' : False,
                'isclass' : False,
                'isfunction' : False,
                'istextfile' : False,
                'iszcmlfile' : False,
                'signature' : None
                }
            entry['url'] = "%s/%s" % (self.context.path.replace('.','/'), name)
            if hasattr(obj,"getDocString"):
                entry['doc'] = self.getDocHeading(obj.getDocString())
            elif hasattr(obj, "getDoc") and isinstance(
                removeAllProxies(obj), InterfaceClass):
                entry['doc'] = self.getDocHeading(obj.getDoc())
            if isinstance(obj, Class):
                entry['isclass'] = True
            elif isinstance(obj, TextFile):
                entry['istextfile'] = True
            elif isinstance(obj, ZCMLFile):
                entry['iszcmlfile'] = True
            elif isinstance(obj,Function):
                entry['isfunction'] = True
                if hasattr(obj, 'getSignature'):
                    entry['signature'] = obj.getSignature()
            elif (isinstance(obj, Module) and
                  os.path.basename(obj.getFileName()) in
                    ['__init.py__', '__init__.pyc', '__init__.pyo']):
                entry['ispackage'] = True
            elif isinstance(obj, Module):
                entry['ismodule'] = True
            elif isinstance(obj, InterfaceClass):
                entry['isinterface'] = True
            entries.append(entry)

        entries.sort(lambda x, y: cmp(x['name'], y['name']))
        return entries

    def update(self):
        self.docgrok_root = self.context._traversal_root
        self.app_root = self.docgrok_root.__parent__
        pass


class DocGrokPackageView(DocGrokView):
    """A view for packages handled by DocGrok."""

    grok.context(DocGrokPackage)
    grok.name('index')


class DocGrokModuleView(DocGrokView):
    """A view for modules handled by DocGrok."""

    grok.context(DocGrokModule)
    grok.name('index')


class DocGrokClassView(DocGrokView):
    """A view for classes handled by DocGrok."""

    grok.context(DocGrokClass)
    grok.name('index')

    def getBases(self):
        return self._listClasses(self.context.apidoc.getBases())

    def getInterfaces(self):
        return self._listClasses(
          [iface for iface in self.context.apidoc.getInterfaces()])

    def getAttributes(self):
        attrs = self.context.getAttributes()
        for a in attrs:
            a['interface'] = self._listClasses([a['interface']])
        return attrs

    def getMethods(self):
        methods = self.context.getMethods()
        for m in methods:
            m['doc'] = renderText(m['attr'].__doc__ or '',
                                  inspect.getmodule(m['attr']))
            m['interface'] = self._listClasses([m['interface']])
        return methods

    def _listClasses(self, classes):
        info = []
        for cls in classes:
            unwrapped_cls = removeAllProxies(cls)
            fullpath = getPythonPath(unwrapped_cls)
            if not fullpath:
                continue
            path, name = fullpath.rsplit('.', 1)
            info.append({
                'path': path or None,
                'path_parts' : self.getPathParts(path) or None,
                'name': name,
                'url': fullpath and fullpath.replace('.','/') or None,
                'doc': self.getDocHeading(cls.__doc__) or None
                })
        return info


class DocGrokInterfaceView(DocGrokClassView):

    grok.context(DocGrokInterface)
    grok.name('index')


class DocGrokGrokApplicationView(DocGrokClassView):

    grok.context(DocGrokGrokApplication)
    grok.name('index')


class DocGrokTextFileView(DocGrokView):

    grok.context(DocGrokTextFile)
    grok.name('index')

    def getContent(self):
        lines = self.context.getContent()
        if self.context.path.endswith('.stx'):
            format = 'zope.source.stx'
        else:
            format = 'zope.source.rest'
        return renderText(lines, format=format)

    def getPackagePathParts(self):
        return self.getPathParts(
            self.context.getPackagePath())



##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup for tests."""

import os
import unittest
from pkg_resources import resource_listdir
from zope.testing import doctest, cleanup
import zope.component.eventtesting
from zope.annotation.attribute import AttributeAnnotations

def setUpZope(test):
    zope.component.eventtesting.setUp(test)
    zope.component.provideAdapter(AttributeAnnotations)

def cleanUpZope(test):
    cleanup.cleanUp()

def suiteFromPackage(name):
    files = resource_listdir(__name__, name)
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename.endswith('_fixture.py'):
            continue
        if filename == '__init__.py':
            continue

        dottedname = 'grok.admin.tests.%s.%s' % (name, filename[:-3])
        test = doctest.DocTestSuite(dottedname,
                                    setUp=setUpZope,
                                    tearDown=cleanUpZope,
                                    optionflags=doctest.ELLIPSIS+
                                    doctest.NORMALIZE_WHITESPACE)

        suite.addTest(test)
    return suite

def pnorm(path):
    """Normalization of paths to use forward slashes. This is needed
    to make sure the tests work on windows.
    """
    return path.replace(os.sep, '/')

def test_suite():
    suite = unittest.TestSuite()
    globs = {'pnorm': pnorm}

    for name in []:
        suite.addTest(suiteFromPackage(name))
    for name in ['docgrok.txt', 'docgrok.py', 'objectinfo.txt', 'utilities.py']:
        suite.addTest(doctest.DocFileSuite(name,
                                           package='grok.admin',
                                           globs=globs,
                                           setUp=setUpZope,
                                           tearDown=cleanUpZope,
                                           optionflags=doctest.ELLIPSIS+
                                           doctest.NORMALIZE_WHITESPACE)
                      )
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""
  >>> getRootFolder()["Manfred"] = Mammoth()

  >>> from zope.app.testing.xmlrpc import ServerProxy
  >>> server = ServerProxy("http://localhost/")

  >>> server.Manfred.stomp()
  'Manfred stomped.'
  >>> server.Manfred.dance()
  "Manfred doesn't like to dance."

"""
import grok


class Mammoth(grok.Model):
    pass


class MammothRPC(grok.XMLRPC):

    def stomp(self):
        return '%s stomped.' % self.context.__name__

    def dance(self):
        return '%s doesn\'t like to dance.' % self.context.__name__



"""
  >>> getRootFolder()["manfred"] = Mammoth()
 
  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/++skin++Basic/manfred/@@cavedrawings")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, world!</h1>
  </body>
  </html>
  
  >>> browser.open("http://localhost/++skin++Rotterdam/manfred/@@moredrawings")
  >>> print browser.contents
  Pretty

  >>> browser.open("http://localhost/++skin++myskin/manfred/@@evenmoredrawings")
  >>> print browser.contents
  Awesome

"""
import grok
from zope.app.basicskin import IBasicSkin
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.app.rotterdam import rotterdam
from zope import interface

grok.layer(IBasicSkin)

class MySkinLayer(grok.IGrokLayer):
    pass

class MySkin(grok.Skin):
    grok.layer(MySkinLayer)

class Mammoth(grok.Model):
    pass

class CaveDrawings(grok.View):
    pass

cavedrawings = grok.PageTemplate("""\
<html>
<body>
<h1>Hello, world!</h1>
</body>
</html>
""")

class MoreDrawings(grok.View):
    grok.layer(rotterdam)

    def render(self):
        return "Pretty"


class EvenMoreDrawings(grok.View):
    grok.layer(MySkinLayer)

    def render(self):
        return "Awesome"



"""
You can't call grok.name on a module:

  >>> import grok.tests.view.nomodulename_fixture
  Traceback (most recent call last):
    ...
  GrokImportError: grok.name can only be used on class level.

"""



"""

Context-determination follows the same rules as for adapters. We just check
whether it's hooked up at all:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: No module-level context for
  <class 'grok.tests.json.nocontext.TestJSON'>, please use grok.context.

"""
import grok

class TestJSON(grok.JSON):
    pass



"""
Only one, either a template, or render() can be specified:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  ConfigurationExecutionError: martian.error.GrokError: Multiple possible ways to render view
  <class 'grok.tests.view.eithertemplateorrender.CavePainting'>.
  It has both a 'render' method as well as an associated template.
  in:
"""
import grok

class Mammoth(grok.Model):
    pass

class CavePainting(grok.View):
    def render(self):
        pass

cavepainting = grok.PageTemplate("nothing")



"""
Using the @grok.require decorator in a view class is not allowed.

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  GrokError: The @grok.require decorator is used for method 'render' in view <class 'grok.tests.security.view_decorator.BogusView'>. It may only be used for XML-RPC methods.


"""

import grok
import zope.interface

class Bogus(grok.Permission):
    grok.name('bogus.perm')

class BogusView(grok.View):
    grok.context(zope.interface.Interface)

    @grok.require('bogus.perm')
    def render(self):
        pass



"""

  >>> grok.testing.grok(__name__)

View with an associated PageTemplate that is referred to using
``grok.template``:

  >>> manfred = Mammoth()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  >>> view = component.getMultiAdapter((manfred, request), name='food')
  >>> print view()
  <html>
  <body>
  ME GROK EAT MAMMOTH!
  </body>
  </html>

"""
import grok
import os.path

class Mammoth(grok.Model):
    pass

class Food(grok.View):
    pass

food = grok.PageTemplate(filename=os.path.join('templatedirectoryname',
                                               'food.pt'))



"""
Templates can also be found in a directory with the same name as the module:

  >>> grok.testing.grok(__name__)
  
  >>> manfred = Mammoth()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  >>> view = component.getMultiAdapter((manfred, request), name='cavepainting')
  >>> print view()
  <html>
  <body>
  A cave painting.
  </body>
  </html>

  >>> view = component.getMultiAdapter((manfred, request), name='food')
  >>> print view()
  <html>
  <body>
  ME GROK EAT MAMMOTH!
  </body>
  </html>

"""
import grok

class Mammoth(grok.Model):
    pass

class CavePainting(grok.View):
    pass

class Food(grok.View):
    pass



"""
A grok.Fields can receive keyword parameters with schema fields. These
should be avaible in the definition order.

  >>> grok.testing.grok(__name__)

  >>> from zope import component
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> view = component.getMultiAdapter((Mammoth(), request), name='edit')
  >>> len(view.form_fields)
  4
  >>> [w.__name__ for w in view.form_fields]
  ['a', 'b', 'g', 'd']

"""
import grok
from zope import schema

class Mammoth(grok.Model):
    pass

class Edit(grok.EditForm):
    form_fields = grok.Fields(
        a = schema.TextLine(title=u"Alpha"),
        b = schema.TextLine(title=u"Beta"),
        g = schema.TextLine(title=u"Gamma"),
        d = schema.TextLine(title=u"Delta"))
    



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok utility functions.
"""

import urllib

import zope.location.location
from zope import component
from zope.traversing.browser.interfaces import IAbsoluteURL
from zope.traversing.browser.absoluteurl import _safe as SAFE_URL_CHARACTERS

from zope.security.checker import NamesChecker, defineChecker
from zope.security.interfaces import IPermission

from martian.error import GrokError, GrokImportError
from martian.util import class_annotation, methods_from_class

def check_adapts(class_):
    if component.adaptedBy(class_) is None:
        raise GrokError("%r must specify which contexts it adapts "
                        "(use grok.adapts to specify)."
                        % class_, class_)

def make_checker(factory, view_factory, permission):
    """Make a checker for a view_factory associated with factory.

    These could be one and the same for normal views, or different
    in case we make method-based views such as for JSON and XMLRPC.
    """
    if permission is not None:
        check_permission(factory, permission)
    if permission is None or permission == 'zope.Public':
        checker = NamesChecker(['__call__'])
    else:
        checker = NamesChecker(['__call__'], permission)
    defineChecker(view_factory, checker)

def check_permission(factory, permission):
    """Check whether a permission is defined.

    If not, raise error for factory.
    """
    if component.queryUtility(IPermission,
                              name=permission) is None:
       raise GrokError('Undefined permission %r in %r. Use '
                       'grok.Permission first.'
                       % (permission, factory), factory)

def get_default_permission(factory):
    """Determine the default permission for a view.
    
    There can be only 0 or 1 default permission.
    """
    permissions = class_annotation(factory, 'grok.require', [])
    if not permissions:
        return None
    if len(permissions) > 1:
        raise GrokError('grok.require was called multiple times in '
                        '%r. It may only be set once for a class.'
                        % factory, factory)

    result = permissions[0]
    return result

def url(request, obj, name=None):
    """Given a request and an object, give the URL.

    Optionally pass a third argument name which gets added to the URL.
    """    
    url = component.getMultiAdapter((obj, request), IAbsoluteURL)()
    if name is None:
        return url
    return url + '/' + urllib.quote(name.encode('utf-8'),
                                    SAFE_URL_CHARACTERS)

def safely_locate_maybe(obj, parent, name):
    """Set an object's __parent__ (and __name__) if the object's
    __parent__ attribute doesn't exist yet or is None.

    If the object provides ILocation, __parent__ and __name__ will be
    set directly.  A location proxy will be returned otherwise.
    """
    if getattr(obj, '__parent__', None) is not None:
        return obj
    # This either sets __parent__ or wraps 'obj' in a LocationProxy
    return zope.location.location.located(obj, parent, name)

def determine_class_directive(directive_name, factory, module_info,
                              default=None):
    directive = class_annotation(factory, directive_name, None)
    if directive is None:
        directive = module_info.getAnnotation(directive_name, None)
    if directive is not None:
        return directive
    return default

def public_methods_from_class(factory):
    return [m for m in methods_from_class(factory) if \
            not m.__name__.startswith('_')]



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok
"""

from zope.interface import implements
from zope.component import adapts
from zope.event import notify
from zope.app.component.hooks import getSite
from zope.lifecycleevent import (
    IObjectCreatedEvent, ObjectCreatedEvent,
    IObjectModifiedEvent, ObjectModifiedEvent,
    IObjectCopiedEvent, ObjectCopiedEvent)

from zope.app.container.contained import (
    IObjectAddedEvent, ObjectAddedEvent,
    IObjectMovedEvent, ObjectMovedEvent,
    IObjectRemovedEvent, ObjectRemovedEvent,
    IContainerModifiedEvent, ContainerModifiedEvent)

from martian import ClassGrokker, InstanceGrokker, GlobalGrokker
from grok.components import Model, Adapter, MultiAdapter, View
from grok.components import XMLRPC, REST, JSON
from grok.components import PageTemplate, PageTemplateFile, Container, Traverser
from grok.components import Site, GlobalUtility, LocalUtility, Annotation
from grok.components import Application, Form, AddForm, EditForm, DisplayForm
from grok.components import Indexes
from grok.components import Permission, Role
from grok.components import Skin, IGrokLayer
from grok.components import RESTProtocol, IRESTLayer
from grok.interfaces import IRESTSkinType
from grok.directive import (context, name, title, template, templatedir,
                            provides, baseclass, global_utility, local_utility,
                            permissions, require, site, layer, direct)
from grok.decorators import subscribe, adapter, implementer
from martian.error import GrokError, GrokImportError

# BBB These two functions are meant for test fixtures and should be
# imported from grok.testing, not from grok.
from grok.testing import grok, grok_component

from grok.formlib import action, AutoFields, Fields
from grok.util import url

# Our __init__ provides the grok API directly so using 'import grok' is enough.
from grok.interfaces import IGrokAPI
from zope.interface import moduleProvides
moduleProvides(IGrokAPI)
__all__ = list(IGrokAPI)



"""
When a package contains a 'static' resource directory, it must not also contain
a module called 'static.py':

  >>> import grok
  >>> grok.testing.grok('grok.tests.static.statichaspy_fixture')
  Traceback (most recent call last):
    ...
  GrokError: A package can not contain both a 'static' resource directory
  and a module named 'static.py'
"""



"""
Since grok.global_utility is a MultipleTimesDirective, there is a list of
GlobalUtilityInfo objects annotated on the module.

  >>> from martian import scan
  >>> from grok.tests.directive import multipletimes
  >>> module_info = scan.module_info_from_module(multipletimes)
  >>> guis = module_info.getAnnotation('grok.global_utility', None)
  >>> guis
  [<grok.directive.GlobalUtilityInfo object at 0x...>,
  <grok.directive.GlobalUtilityInfo object at 0x...>]
  >>> guis[0].factory
  <class 'grok.tests.directive.multipletimes.Club'>
  >>> guis[0].provides
  <InterfaceClass grok.tests.directive.multipletimes.IClub>
  >>> guis[0].name
  'foo'
  >>> guis[1].factory
  <class 'grok.tests.directive.multipletimes.Cave'>
  >>> guis[1].provides is None
  True
  >>> guis[1].name
  u''
  
"""
import grok
from zope import interface

class IClub(interface.Interface):
    pass

class ICave(interface.Interface):
    pass

class Club(object):
    grok.implements(IClub)

class Cave(object):
    grok.implements(ICave)

grok.global_utility(Club, provides=IClub, name='foo')
grok.global_utility(Cave)



import sys

from zope.interface import implements
from zope.interface.interfaces import IMethod, IInterface

from zope.app.catalog.field import FieldIndex
from zope.app.catalog.text import TextIndex
from zc.catalog.catalogindex import SetIndex

from martian.error import GrokError, GrokImportError
from martian.util import frame_is_class

from grok.interfaces import IIndexDefinition

class IndexDefinition(object):
    implements(IIndexDefinition)

    index_class = None

    def __init__(self, *args, **kw):
        frame = sys._getframe(1)
        if not frame_is_class(frame):
            raise GrokImportError(
                "%r can only be instantiated on class level." % self.__class__)
        # store any extra parameters to pass to index later
        self._args = args
        self._attribute = kw.pop('attribute', None)
        self._kw = kw

    def setup(self, catalog, name, context, module_info):
        if self._attribute is not None:
            field_name = self._attribute
        else:
            field_name = name

        if IInterface.providedBy(context):
            try:
                method = context[field_name]
            except KeyError:
                raise GrokError("grok.Indexes in %r refers to an attribute or "
                                "method %r on interface %r, but this does not "
                                "exist." % (module_info.getModule(),
                                            field_name, context), None)
            call = IMethod.providedBy(method)
        else:
            call = callable(getattr(context, field_name, None))
            context = None # no interface lookup
        catalog[name] = self.index_class(field_name=field_name,
                                         interface=context,
                                         field_callable=call,
                                         *self._args, **self._kw)

class Field(IndexDefinition):
    index_class = FieldIndex

class Text(IndexDefinition):
    index_class = TextIndex

class Set(IndexDefinition):
    index_class = SetIndex



"""
A grok.Model may implement a schema that inherits from another one:

  >>> grok.testing.grok(__name__)
  >>> manfred = Mammoth()

  >>> from zope import component
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()

Without AutoFields, just a simple edit form:

  >>> view = component.getMultiAdapter((manfred, request), name='edit')
  >>> len(view.form_fields)
  3
  >>> [w.__name__ for w in view.form_fields]
  ['name', 'size', 'speciality']

With AutoFields:

  >>> view = component.getMultiAdapter((manfred, request), name='edit2')
  >>> len(view.form_fields)
  3
  >>> [w.__name__ for w in view.form_fields]
  ['name', 'size', 'speciality']

  >>> antimanfred = YetAnotherMammoth()
  >>> view = component.getMultiAdapter((antimanfred, request), name='edit3')
  >>> len(view.form_fields)
  3
  >>> [w.__name__ for w in view.form_fields]
  ['name', 'size', 'speciality']
"""
import grok
from zope import interface, schema

class IMammoth(interface.Interface):
    name = schema.TextLine(title=u"Name")
    size = schema.TextLine(title=u"Size", default=u"Quite normal")

class ISpecialMammoth(IMammoth):
    speciality = schema.TextLine(title=u"Speciality")

class Mammoth(grok.Model):
    interface.implements(ISpecialMammoth)

class Edit(grok.EditForm):
    grok.context(Mammoth)

class Edit2(grok.EditForm):
    grok.context(Mammoth)

    form_fields = grok.AutoFields(Mammoth)

# situation where subclass implements something on top of base class
class AnotherMammoth(grok.Model):
    interface.implements(IMammoth)

class YetAnotherMammoth(AnotherMammoth):
    interface.implements(ISpecialMammoth)

class Edit3(grok.EditForm):
    grok.context(YetAnotherMammoth)
    



##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  
We fetch the documentation page, which should give us a tiny overview
over documentation:

  >>> browser.open("http://localhost/docgrok")
  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ... Welcome to DocGrok...
  ...

On the documentation page there should be a link to the ZODB root
folder:

  >>> root_link = browser.getLink('ZODB root folder')
  >>> root_link
  <Link text='ZODB root folder' url='http://localhost/@@inspect.html'>

The root folder got no name:

  >>> root_link.click()
  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ... <span>&lt;unnamed object&gt;</span>
  ...

and is of type Folder.

  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ... <span ...>...<a ...>Folder</a> object at ...&gt;</span>
  ...

It's class documentation should be linked in the head of page:

  >>> browser.getLink('Folder').url
  'http://localhost/docgrok/zope/app/folder/folder/Folder/'

We also get the docstring of the root folder:

  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ...<p>The standard Zope Folder implementation.</p>
  ...
  
A checkbox gives us control over private members and attributes of the
object displayed:

  >>> checkbox = browser.getControl('Show private attributes')
  >>> checkbox
  <ItemControl name='show_private' type='checkbox' optionValue='on' selected=False>

By default the checkbox is not selected. Therefore we check for an
arbitrary private method to be displayed or not. For example the
``__dict__`` method. By default no __dict__ method will be displayed:

  >>> '__dict__' in browser.contents
  False

Now let's tick the checkbox and update the view:

  >>> checkbox.selected = True
  >>> checkbox.selected
  True

  >>> browser.getControl('update').click()

Now the private method should be displayed:

  >>> '__dict__' in browser.contents
  True

Here we go :-)

Okay, now let's examine the displayed data a bit. We are currently the
object browser's view for the root folder. The root folder got no
parent, which should be displayed:

  >>> 'No parent object' in browser.contents
  True

One of the base classes of the root folder is the class
``persistent.Persistent``. We not only want that displayed but also a
link to the class documentation of that class:

  >>> link = browser.getLink("Persistent'")
  >>> link.url
  'http://localhost/docgrok/persistent/Persistent'

The same for interfaces. The root folder implements
``persistent.interfaces.IPersistent``:w

  >>> link = browser.getLink('IPersistent')
  >>> link.url
  'http://localhost/docgrok/persistent/interfaces/IPersistent/'

Now the attributes and properties. The root folder got an attribute
``data``:

  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ...<h3 ...>
  ...Attributes
  ...</h3>
  ... <span>data</span>
  ... <div>
  ...  value:
  ...   <a href="http://localhost/docgrok-obj/data/@@inspect.html">&lt;BTrees.OOBTree.OOBTree object at ...&gt;</a>
  ... </div>
  ...
  


"""



"""
  >>> getRootFolder()["manfred"] = Mammoth()

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/manfred/@@painting")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, world!</h1>
  </body>
  </html>

"""
import grok

class Mammoth(grok.Model):
    pass

class Painting(grok.View):
    pass

painting = grok.PageTemplate("""\
<html>
<body>
<h1>Hello, world!</h1>
</body>
</html>
""")



"""
Forms cannot define a render method. Here we show the case where the
EditForm has no explicit template associated with it:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  ...
  GrokError: It is not allowed to specify a custom 'render' method for
  form <class 'grok.tests.form.norender2.Edit'>. Forms either use the default
  template or a custom-supplied one.
  
"""

import grok

class Mammoth(grok.Model):
    pass

class Edit(grok.EditForm):
    # not allowed to have a render method
    def render(self):
        return "this cannot be"



"""
Forms can implement an update() method that will be called before any
form processing has happened:

  >>> getRootFolder()["manfred"] = Mammoth()

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/manfred/edit")
  >>> browser.getControl(name="form.name").value = "Ellie"
  >>> browser.getControl("Apply").click()

  >>> browser.open("http://localhost/manfred")
  >>> print browser.contents
  Ellie, the Mammoth reports: The form's update() was called and my name was Manfred.

A form's update() method can issue a redirect.  In that case, the form
won't proceed to do any form processing nor rendering:

  >>> browser.open("http://localhost/manfred/editredirect")
  >>> browser.getControl(name="form.name").value = "Mallie"
  >>> browser.getControl("Apply").click()
  >>> print browser.url
  http://localhost/manfred/index

Because of the redirect, no changes happened to the edited object:

  >>> print browser.contents
  Ellie, the Mammoth reports: The form's update() was called and my name was Manfred.

A form's update() method may also take arbitrary parameters that will
be filled with values from the request (such as form values):

  >>> browser.open("http://localhost/manfred/editupdatewitharguments")
  >>> browser.getControl(name="report").value = "Request argument dispatch to update() works."
  >>> browser.getControl(name="form.name").value = "Mallie"
  >>> browser.getControl("Apply").click()

  >>> browser.open("http://localhost/manfred")
  >>> print browser.contents
  Mallie, the Mammoth reports: Request argument dispatch to update() works.

"""
import grok
from zope import schema

from zope.interface import Interface, implements

class IMammoth(Interface):
    name = schema.TextLine(title=u"Name")

class Mammoth(grok.Model):
    implements(IMammoth)
    
    name = u'Manfred'

class Index(grok.View):

    def render(self):
        return "%s, the Mammoth reports: %s" % (self.context.name,
                                                self.context.report)

class Edit(grok.EditForm):

    def update(self):
        self.context.report = ("The form's update() was called and my name "
                               "was %s." % self.context.name)

class EditRedirect(grok.EditForm):

    def update(self):
        # redirect upon form submit so that no changes are ever saved
        if 'form.name' in self.request:
            self.redirect(self.url('index'))

class EditUpdateWithArguments(grok.EditForm):

    def update(self, report=None):
        if report is not None:
            self.context.report = report

editupdatewitharguments = grok.PageTemplate("""
<html>
<body>
<form action="" tal:attributes="action request/URL">
  <input type="text" name="report" />
  <div tal:repeat="widget view/widgets" tal:content="structure widget" />
  <div tal:repeat="action view/actions" tal:content="structure action/render" />
</form>
</body>
</html>
""")



##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""

  >>> from zope.testbrowser.testing import Browser
  >>> browser = Browser()
  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  
We fetch the standard page, which should provide us a menu to get all
installable grok applications/components.

  >>> browser.open("http://localhost/")
  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ...      <legend>Add application</legend>
  ...

The opening screen should inform us, that there are no applications
installed yet:

  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ... <p ...>Currently no working...applications are installed.</p>
  ...

We are able to add a mammoth manager...

  >>> subform = browser.getForm(name='MammothManager')
  >>> subform.getControl('Name your new app:').value = 'my-mammoth-manager'
  >>> subform.getControl('Create').click()

  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ...<legend>Installed applications</legend>
  ...
  ...<a href="http://localhost/my-mammoth-manager">
  ...

Launch the added mammoth manager

  >>> mylink = browser.getLink('my-mammoth-manager (MammothManager)').click()
  >>> print browser.contents
  Let's manage some mammoths!

  >>> print browser.url
  http://localhost/my-mammoth-manager

We can go to the object browser for every installed application:

  >>> browser.open("http://localhost/applications")
  >>> browser.getLink('object browser').click()
  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ...<span ...>...<a href=...>MammothManager</a> object at ...></span>
  ... 

We are able to delete installed mammoth-managers

  >>> browser.open("http://localhost/applications")
  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ... <legend>Installed applications</legend>
  ...
  >>> ctrl = browser.getControl(name='items')
  >>> ctrl.getControl(value='my-mammoth-manager').selected = True
  >>> browser.getControl('Delete Selected').click()
  >>> print browser.contents
  <html xmlns="http://www.w3.org/1999/xhtml">
  ...
  ... <p ...>Currently no working applications...are installed.</p>
  ...
  ...<legend>Add application</legend>
  ...

"""

import grok

class MammothManager(grok.Application, grok.Container):
    """A mammoth manager"""
    pass

class Index(grok.View):#

    def render(self):
        return u"Let's manage some mammoths!"



import grok

from zope import component

from zope.traversing.namespace import skin
from zope.interface import Interface
from zope.interface.interfaces import IInterface
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.interfaces.http import IHTTPRequest
from zope.app.publication.http import MethodNotAllowed
import zope.location

from grok.interfaces import IRESTSkinType

class RestPublisher(zope.location.Location):
    grok.implements(IBrowserPublisher)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.__parent__ = self.context

class GrokMethodNotAllowed(MethodNotAllowed):
    pass

class MethodNotAllowedView(grok.MultiAdapter):
    grok.adapts(GrokMethodNotAllowed, IHTTPRequest)
    grok.name('index.html')
    grok.implements(Interface)
    
    def __init__(self, error, request):
        self.error = error
        self.request = request
        self.allow = self._getAllow()
        
    def _getAllow(self):
        allow = []
        for method in ['GET', 'PUT', 'POST', 'DELETE']:
            view = component.queryMultiAdapter(
                (self.error.object, self.error.request),
                name=method)
            if view is not None:
                is_not_allowed = getattr(view, 'is_not_allowed', False)
                if not is_not_allowed:
                    allow.append(method)
        allow.sort()
        return allow
    
    def __call__(self):
        self.request.response.setHeader('Allow', ', '.join(self.allow))
        self.request.response.setStatus(405)
        return 'Method Not Allowed'
    
class rest_skin(skin):
    skin_type = IRESTSkinType

class DefaultRest(grok.REST):
    grok.context(Interface)
    grok.layer(grok.IRESTLayer)

class NotAllowedREST(grok.REST):
    """These are registered for everything by default to cause the correct
    errors.

    Any more specific REST view overrides this.
    """
    grok.layer(grok.IRESTLayer)
    grok.context(Interface)

    is_not_allowed = True
    
    def GET(self):
        raise GrokMethodNotAllowed(self.context, self.request)
            
    def POST(self):
        raise GrokMethodNotAllowed(self.context, self.request)
            
    def PUT(self):
        raise GrokMethodNotAllowed(self.context, self.request)
    
    def DELETE(self):
        raise GrokMethodNotAllowed(self.context, self.request)
            



"""
Testing the plugging in of a template language

  >>> grok.testing.grok(__name__)
  
  >>> cave = Cave()
  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  
  # The inline template should work:
  >>> view = component.getMultiAdapter((cave, request), name='sebaayeni')
  >>> print view()
  <html><body>Sebaayeni is in South Africa</body></html>

  # And the inline file template:
  >>> view = component.getMultiAdapter((cave, request), name='lascaux')
  >>> print view()
  <html><body>Lascaux is in France</body></html>

  # And the template directory template:
  >>> view = component.getMultiAdapter((cave, request), name='kakadu')
  >>> print view()
  <html><body>Kakadu is in Australia</body></html>

  # We should be able to extend the namespac in the view and 
  >>> view = component.getMultiAdapter((cave, request), name='sierra')
  >>> print view()
  <html><body>Sierra de San Fransisco is in Mexico</body></html>

"""
import grok, os

# Dummy template language:
class MyTemplate(object):
    
    def __init__(self, text):
        self._text = text
            
    def render(self, **kw):
        # Silliest template language ever:
        return self._text % kw

class MyPageTemplate(grok.components.GrokTemplate):

    def setFromString(self, string):
        self._template = MyTemplate(string)

    def setFromFilename(self, filename, _prefix=None):
        file = open(os.path.join(_prefix, filename))
        self._template = MyTemplate(file.read())

    def namespace(self, view):
        # I'll override the default namespace here for testing:
        return {'middle_text': 'is in'}

    def render(self, view):
        return self._template.render(**self.getNamespace(view))

class MyPageTemplateFactory(grok.GlobalUtility):

    grok.implements(grok.interfaces.ITemplateFileFactory)
    grok.name('mtl')

    def __call__(self, filename, _prefix=None):
        return MyPageTemplate(filename=filename, _prefix=_prefix)

class Cave(grok.Model):
    pass

class Sebaayeni(grok.View):
    pass
    
sebaayeni = MyPageTemplate('<html><body>Sebaayeni is in South Africa</body></html>')

class Lascaux(grok.View):
    pass
    
lascaux = MyPageTemplate(filename='lascaux.html')

class Kakadu(grok.View):
    pass

class Sierra(grok.View):
    
    def namespace(self):
        return {'cave': 'Sierra de San Fransisco',
                'country': 'Mexico'}



"""
Anything can be registered as a local utility. If it implements a single
interface, there is no need to specify which interface it provides.

In this test, the utility implements more than one interface, so it cannot be
registered as a local utility.

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.utility.local_implementsmany.Fireplace'>
  is implementing more than one interface (use grok.provides to specify
  which one to use).

"""

import grok
from zope import interface

class IHome(interface.Interface):
    pass

class IFireplace(interface.Interface):
    pass

class Fireplace(object):
    interface.implements(IHome, IFireplace)

class Cave(grok.Model, grok.Site):
    grok.local_utility(Fireplace)



##############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok interfaces
"""
from zope import interface, schema
from zope.publisher.interfaces.browser import IBrowserPage, IBrowserView
from zope.formlib.interfaces import reConstraint
from zope.interface.interfaces import IInterface

class IGrokBaseClasses(interface.Interface):
    ClassGrokker = interface.Attribute("Base class to define a class "
                                       "grokker.")
    InstanceGrokker = interface.Attribute("Base class to define an "
                                          "instance grokker.")
    ModuleGrokker = interface.Attribute("Base class to define a "
                                        "module grokker.")
    Model = interface.Attribute("Base class for persistent content objects "
                                "(models).")
    Container = interface.Attribute("Base class for containers.")
    Site = interface.Attribute("Mixin class for sites.")
    Application = interface.Attribute("Base class for applications.")
    Adapter = interface.Attribute("Base class for adapters.")
    MultiAdapter = interface.Attribute("Base class for multi-adapters.")
    Annotation = interface.Attribute("Base class for persistent annotations.")
    GlobalUtility = interface.Attribute("Base class for global utilities.")
    LocalUtility = interface.Attribute("Base class for local utilities.")
    View = interface.Attribute("Base class for browser views.")
    XMLRPC = interface.Attribute("Base class for XML-RPC methods.")
    JSON = interface.Attribute("Base class for JSON methods.")
    REST = interface.Attribute("Base class for REST views.")
    Traverser = interface.Attribute("Base class for custom traversers.")
    Form = interface.Attribute("Base class for forms.")
    AddForm = interface.Attribute("Base class for add forms.")
    EditForm = interface.Attribute("Base class for edit forms.")
    DisplayForm = interface.Attribute("Base class for display forms.")
    Indexes = interface.Attribute("Base class for catalog index definitions.")
    Layer = interface.Attribute("Base interface for layers.")
    Skin = interface.Attribute("Base class for skin.")


class IGrokErrors(interface.Interface):

    def GrokError(message, component):
        """Error indicating that a problem occurrend during the
        grokking of a module (at "grok time")."""

    def GrokImportError(*args):
        """Error indicating a problem at import time."""


class IGrokDirectives(interface.Interface):

    def implements(*interfaces):
        """Declare that a class implements the given interfaces."""

    def adapts(*classes_or_interfaces):
        """Declare that a class adapts objects of the given classes or
        interfaces."""

    def context(class_or_interface):
        """Declare the context for views, adapters, etc.

        This directive can be used on module and class level.  When
        used on module level, it will set the context for all views,
        adapters, etc. in that module.  When used on class level, it
        will set the context for that particular class."""

    def name(name):
        """Declare the name of a view or adapter/multi-adapter.

        This directive can only be used on class level."""

    def layer(layer):
        """Declare the layer for the view.

        This directive acts as a contraint on the 'request' of
        grok.View. This directive can only be used on class level."""

    def skin(skin):
        """Declare this layer as a named skin.

        This directive can only be used on class level."""

    def template(template):
        """Declare the template name for a view.

        This directive can only be used on class level."""

    def templatedir(directory):
        """Declare a directory to be searched for templates.

        By default, grok will take the name of the module as the name
        of the directory.  This can be overridden using
        ``templatedir``."""

    def provides(interface):
        """Explicitly specify with which interface a component will be
        looked up."""

    def baseclass():
        """Mark this class as a base class.

        This means it won't be grokked, though if it's a possible context,
        it can still serve as a context.
        """

    def global_utility(factory, provides=None, name=u''):
        """Register a global utility.

        factory - the factory that creates the global utility
        provides - the interface the utility should be looked up with
        name - the name of the utility
        """

    def local_utility(factory, provides=None, name=u'',
                      setup=None, public=False, name_in_container=None):
        """Register a local utility.

        factory - the factory that creates the local utility
        provides - the interface the utility should be looked up with
        name - the name of the utility
        setup - a callable that receives the utility as its single argument,
                it is called after the utility has been created and stored
        public - if False, the utility will be stored below ++etc++site
                 if True, the utility will be stored directly in the site.
                 The site should in this case be a container.
        name_in_container - the name to use for storing the utility
        """

    def permissions(permissions):
        """Specify the permissions that comprise a role.
        """

    def require(permission):
        """Protect a view class or an XMLRPC method with ``permision``.

        ``permission`` must already be defined, e.g. using
        grok.Permission.

        grok.require can be used as a class-level directive or as a
        method decorator."""

    def site(class_or_interface):
        """Specifies the site that an indexes definition is for.

        It can only be used inside grok.Indexes subclasses.
        """


class IGrokDecorators(interface.Interface):

    def subscribe(*classes_or_interfaces):
        """Declare that a function subscribes to an event or a
        combination of objects and events."""

    def action(label, **options):
        """Decorator that defines an action factory based on a form
        method. The method receives the form data as keyword
        parameters."""


class IGrokEvents(interface.Interface):

    IObjectCreatedEvent = interface.Attribute("")

    ObjectCreatedEvent = interface.Attribute("")

    IObjectModifiedEvent = interface.Attribute("")

    ObjectModifiedEvent = interface.Attribute("")

    IObjectCopiedEvent = interface.Attribute("")

    ObjectCopiedEvent = interface.Attribute("")

    IObjectAddedEvent = interface.Attribute("")

    ObjectAddedEvent = interface.Attribute("")

    IObjectMovedEvent = interface.Attribute("")

    ObjectMovedEvent = interface.Attribute("")

    IObjectRemovedEvent = interface.Attribute("")

    ObjectRemovedEvent = interface.Attribute("")

    IContainerModifiedEvent = interface.Attribute("")

    ContainerModifiedEvent = interface.Attribute("")


class IGrokAPI(IGrokBaseClasses, IGrokDirectives, IGrokDecorators,
               IGrokEvents, IGrokErrors):

    def grok(dotted_name):
        """Grok a module or package specified by ``dotted_name``."""

    def grok_component(name, component, context=None, module_info=None,
                       templates=None):
        """Grok an arbitrary object. Can be useful during testing.

        name - the name of the component (class name, or global instance name
               as it would appear in a module).
        component - the object (class, etc) to grok.
        context - the context object (optional).
        module_info - the module being grokked (optional).
        templates - the templates registry (optional).

        Note that context, module_info and templates might be required
        for some grokkers which rely on them.
        """

    def url(request, obj, name=None):
        """Generate the URL to an object with optional name attached.
        """

    def notify(event):
        """Send ``event`` to event subscribers."""

    def getSite():
        """Get the current site."""

    def PageTemplate(template):
        """Create a Grok PageTemplate object from ``template`` source
        text.  This can be used for inline PageTemplates."""

    def PageTemplateFile(filename):
        """Create a Grok PageTemplate object from a file specified by
        ``filename``.  It will be treated like an inline template
        created with ``PageTemplate``."""

    def Fields(*args, **kw):
        """Return a list of formlib fields based on interfaces and/or schema
        fields."""

    def AutoFields(context):
        """Return a list of fields for context autogenerated by grok.
        """

    def action(label, actions=None, **options):
        """grok-specific action decorator.
        """

    IRESTSkinType = interface.Attribute('The REST skin type')

class IGrokView(IBrowserPage, IBrowserView):
    """Grok views all provide this interface."""

    context = interface.Attribute('context', "Object that the view presents.")

    request = interface.Attribute('request', "Request that the view was looked"
                                  "up with.")

    response = interface.Attribute('response', "Response object that is "
                                   "associated with the current request.")

    static = interface.Attribute('static', "Directory resource containing "
                                 "the static files of the view's package.")

    def redirect(url):
       """Redirect to given URL"""

    def url(obj=None, name=None):
        """Construct URL.

        If no arguments given, construct URL to view itself.

        If only obj argument is given, construct URL to obj.

        If only name is given as the first argument, construct URL
        to context/name.

        If both object and name arguments are supplied, construct
        URL to obj/name.
        """

    def update(**kw):
        """This method is meant to be implemented by grok.View
        subclasses.  It will be called *before* the view's associated
        template is rendered and can be used to pre-compute values
        for the template.

        update() can take arbitrary keyword parameters which will be
        filled in from the request (in that case they *must* be
        present in the request)."""

    def render(**kw):
        """A view can either be rendered by an associated template, or
        it can implement this method to render itself from Python.
        This is useful if the view's output isn't XML/HTML but
        something computed in Python (plain text, PDF, etc.)

        render() can take arbitrary keyword parameters which will be
        filled in from the request (in that case they *must* be
        present in the request)."""

    def application_url(name=None):
        """Return the URL of the closest application object in the
        hierarchy or the URL of a named object (``name`` parameter)
        relative to the closest application object.
        """

    def flash(message, type='message'):
        """Send a short message to the user."""


class IGrokForm(IGrokView):
    """Grok form API, inspired by zope.formlib's IFormBaseCustomization.

    We explicitly don't inherit from IFormBaseCustomization because
    that would imply ISubPage with another definition of update() and
    render() than IGrokView has.
    """

    prefix = schema.ASCII(
        constraint=reConstraint(
            '[a-zA-Z][a-zA-Z0-9_]*([.][a-zA-Z][a-zA-Z0-9_]*)*',
            "Must be a sequence of not-separated identifiers"),
        description=u"""Page-element prefix

        All named or identified page elements in a subpage should have
        names and identifiers that begin with a subpage prefix
        followed by a dot.
        """,
        readonly=True,
        )

    def setPrefix(prefix):
        """Update the subpage prefix
        """

    label = interface.Attribute("A label to display at the top of a form")

    status = interface.Attribute(
        """An update status message

        This is normally generated by success or failure handlers.
        """)

    errors = interface.Attribute(
        """Sequence of errors encountered during validation
        """)

    form_result = interface.Attribute(
        """Return from action result method
        """)

    form_reset = interface.Attribute(
        """Boolean indicating whether the form needs to be reset
        """)

    form_fields = interface.Attribute(
        """The form's form field definitions

        This attribute is used by many of the default methods.
        """)

    widgets = interface.Attribute(
        """The form's widgets

        - set by setUpWidgets

        - used by validate
        """)

    def setUpWidgets(ignore_request=False):
        """Set up the form's widgets.

        The default implementation uses the form definitions in the
        form_fields attribute and setUpInputWidgets.

        The function should set the widgets attribute.
        """

    def validate(action, data):
        """The default form validator

        If an action is submitted and the action doesn't have it's own
        validator then this function will be called.
        """

    template = interface.Attribute(
        """Template used to display the form
        """)

    def resetForm():
        """Reset any cached data because underlying content may have changed
        """

    def error_views():
        """Return views of any errors.

        The errors are returned as an iterable.
        """

    def applyData(obj, **data):
        """Save form data to an object.

        This returns a dictionary with interfaces as keys and lists of
        field names as values to indicate which fields in which
        schemas had to be changed in order to save the data.  In case
        the method works in update mode (e.g. on EditForms) and
        doesn't have to update an object, the dictionary is empty.
        """

class IREST(interface.Interface):
    context = interface.Attribute("Object that the REST handler presents.")

    request = interface.Attribute("Request that REST handler was looked"
                                  "up with.")
    
    body = interface.Attribute(
        """The text of the request body.""")

class IApplication(interface.Interface):
    """Marker-interface for grok application factories.

    Used to register applications as utilities to look them up and
    provide a list of grokked applications.
    """

class IIndexDefinition(interface.Interface):
    """Define an index for grok.Indexes.
    """

    def setup(catalog, name, context):
        """Set up index called name in given catalog.

        Use name for index name and attribute to index. Set up
        index for interface or class context.
        """

class IRESTSkinType(IInterface):
    """Skin type for REST requests.
    """

class ITemplateFileFactory(interface.Interface):
    """Utility that generates templates from files in template directories. 
    """
    
    def __call__(filename, _prefix=None):
        """Creates an ITemplate from a file
        
        _prefix is the directory the file is located in
        """

class ITemplate(interface.Interface):
    """Template objects
    """
    
    def _initFactory(factory):
        """Template language specific initializations on the view factory."""
        
    def render(view):
        """Renders the template"""




##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Helper functions for grok admin.
"""
import re
from zope.tal.taldefs import attrEscape
from urlparse import urlparse, urlunparse

def getPathLinksForObject(obj, root_url=''):
    """Given an object, this function returns HTML code with links to
    documentation.

    The object must provide a string representation like 'foo.blah
    object at 0x9999999'. Returned is then a string, where 'foo' and
    'blah' are embedded in HTML links to docgrok documentation for foo
    and foo.blah.

    The (optional) ``root_url`` is used to create the links to docgrok
    documentation. It is expected to be the URL, which can generate
    docgrok documentation by appending '/docgrok' to the URL.

    We can use ObjectInfo objects to check this:

      >>> from grok.admin.objectinfo import ObjectInfo
      >>> obj = ObjectInfo(None)
      >>> obj
      <grok.admin.objectinfo.ObjectInfo object at ...>

    Obviously we have a string representation of the required form
    here. So we can get HTML with links to the documentation for
    ``grok``, ``grok.admin`` and so on.
    
      >>> from grok.admin.utilities import getPathLinksForObject
      >>> link = getPathLinksForObject(obj)
      >>> link
      "&lt;<a href='/docgrok/grok/'>grok</a>... object at ..."

    We got a link to the ``grok`` documentation. Also links to
    ``grok.admin``, ``grok.admin.objectinfo`` and
    ``grok.admin.objectinfo.ObjectInfo`` are provided:

      >>> link
      "&lt;...<a href='/docgrok/grok/admin/'>admin</a>... object at ..."

    If we provide a root_url, we will find it in the links:

      >>> link = getPathLinksForObject(obj, 'http://localhost:8080')
      >>> link
      "&lt;<a href='http://localhost:8080/docgrok/grok/'>grok</a>..."

    If no dotted path is included in objects strings representation, a
    simple string without links is returned:
    
      >>> getPathLinksForObject(None)
      "'None'"

    HTML entities should be encoded. We set up a site-manager to get
    an 'illegal' object representation including regular expression
    chars ('+') and no dotted path:

      >>> from zope.app.folder import rootFolder
      >>> root = rootFolder()
      >>> from zope.app.component import site
      >>> sm = site.LocalSiteManager(root)
      >>> root.setSiteManager(sm)
      >>> sm
      <LocalSiteManager ++etc++site>

    This is a strange object identifier. Anyway:

      >>> getPathLinksForObject(sm)
      "'&lt;LocalSiteManager ++etc++site&gt;'"
      
    """
    r_exp = re.compile("'<(.+)( object at .*)>'")
    
    raw = `str(obj)`
    match = r_exp.match(raw)
    if match is None:
        return attrEscape(raw)

    result = "&lt;"
    url = root_url + '/docgrok/'
    for part in match.group(1).split('.'):
        url = url + part + '/'
        result += "<a href='%s'>%s</a>." % (url, part)
    if len(result) and result[-1] == '.':
        result = "%s%s&gt;" % (result[:-1], match.group(2))
        return result
    return raw

def getPathLinksForClass(klass, root_url=''):
    """Given a class or classlike object, this function returns HTML
    code with links to documentation.

    The klass object must provide a string representation like '<class
    foo.Bar>'. Returned is then a string, where 'foo' and
    'Bar' are embedded in HTML links to docgrok documentation for foo
    and foo.Bar.

    The (optional) ``root_url`` is used to create the links to docgrok
    documentation. It is expected to be the URL, which can generate
    docgrok documentation by appending '/docgrok' to the URL.

    We can use class ObjectInfo to check this:

      >>> from grok.admin.objectinfo import ObjectInfo
      >>> ObjectInfo
      <class 'grok.admin.objectinfo.ObjectInfo'>

      >>> from grok.admin.utilities import getPathLinksForClass
      >>> htmlcode = getPathLinksForClass(ObjectInfo)
      >>> htmlcode
      "&lt;class '<a href='/docgrok/grok/'>grok</a>...'&gt;"

    When we provide a root_url the link will include it in the
    href-attribute:

      >>> getPathLinksForClass(ObjectInfo, 'http://localhost')
      "&lt;class '<a href='http://localhost/docgrok/grok/'>grok</a>...'&gt;"

    If the class does not provide an appropriate string
    representation, we will get the representation without any links:

      >>> getPathLinksForClass(None, 'http://localhost')
      "'None'"

    This also works with 'class-like' objects, for instance interfaces
    and their interface-classes:

      >>> from zope.app.folder import rootFolder
      >>> from zope.interface import providedBy
      >>> root = rootFolder()
      >>> iface = list(providedBy(root))[0]
      >>> iface
      <InterfaceClass zope.app.folder.interfaces.IRootFolder>

      >>> getPathLinksForClass(iface)
      "&lt;InterfaceClass '<a href='/docgrok/zope/'>zope</a>...'&gt;"

    HTML entities should be encoded. We set up a site-manager to get
    an 'illegal' object representation including regular expression
    chars ('+') and no dotted path:

      >>> from zope.app.folder import rootFolder
      >>> root = rootFolder()
      >>> from zope.app.component import site
      >>> sm = site.LocalSiteManager(root)
      >>> root.setSiteManager(sm)
      >>> sm
      <LocalSiteManager ++etc++site>

    This is a strange object identifier. Anyway:

      >>> getPathLinksForClass(sm)
      "&lt;LocalSiteManager '<a href='/docgrok/++etc++site/'>...</a>'&gt;"

    """
    r_exp = re.compile(".*<(.*) '?(.+)'?(.*)>.*")
    raw = `str(klass)`
    match = r_exp.match(raw)
    if match is None:
        return attrEscape(raw)

    result = "&lt;%s '" % (match.group(1),)
    url = root_url + '/docgrok/'
    for part in match.group(2).split('.'):
        url = "%s%s/" % (url, part)
        result += "<a href='%s'>%s</a>." % (url, part)
    if len(result) and result[-1] == '.':
        result = "%s'%s&gt;" % (result[:-1], match.group(3))
        return result
    return raw

def getPathLinksForDottedName(name, root_url=''):
    """
    """
    if name is None:
        return ''
    result = ''
    url = root_url + '/docgrok/'
    for part in name.split('.'):
        url = "%s%s/" % (url, part)
        result += "<a href='%s'>%s</a>." % (url, part)
    if len(result) and result.endswith('.'):
        result = result[:-1]
        return result
    return name

def isContainingEvilRegExpChars(strval):
    """Check whether a string contains evil chars.

    'Evil' with respect to regular expressions is a string, that
    contains chars, with a special meaning in regular expressions.

    We indeed must provide a string:

       >>> from grok.admin.utilities import isContainingEvilRegExpChars
       >>> isContainingEvilRegExpChars(None)
       Traceback (most recent call last):
       ...
       TypeError: expected string or buffer

       >>> isContainingEvilRegExpChars('foo')
       False

       >>> isContainingEvilRegExpChars('foo++etc++bar')
       True

       >>> isContainingEvilRegExpChars('foo*bar')
       True

    """
    evil_chars = re.compile('.*(\*|\+|\(|\)|\{|\}).*')
    if evil_chars.match(strval):
        return True
    return False


def getParentURL(url):
    """Compute the parent URL for an object described by URL.

       >>> from grok.admin.utilities import getParentURL
       >>> getParentURL('http://foo:8080/myobj')
       'http://foo:8080/'

       >>> getParentURL('http://foo:8080/myfoo/mybar')
       'http://foo:8080/myfoo/'

    We want an URL always to end with a slash:

       >>> getParentURL('http://foo:8080')
       'http://foo:8080/'

    """
    url_list = list(urlparse(url))
    path = url_list[2]
    if path.endswith('/'):
        path = path[:-1]
    path = path.rsplit('/', 1)[0] + '/'
    url_list[2] = path
    return urlunparse(url_list)



"""
  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: Cannot determine which interface to use for utility registration of
  <class 'grok.tests.utility.local_implementsnone2.Fireplace'> in site
  <class 'grok.tests.utility.local_implementsnone2.Cave'>. It implements
  an interface that is a specialization of an interface implemented
  by grok.LocalUtility. Specify the interface by either using grok.provides on
  the utility or passing 'provides' to grok.local_utility.

"""

import grok
from zope import interface
import persistent

class ISpecialPersistent(persistent.interfaces.IPersistent):
    pass

class Fireplace(grok.LocalUtility):
    grok.implements(ISpecialPersistent)

class Cave(grok.Model, grok.Site):
    grok.local_utility(Fireplace)



"""
  >>> util.class_annotation_list(B, 'grok.foo', None)
  [5, 7]
  >>> util.class_annotation_list(B2, 'grok.foo', None)
  [5]
  >>> util.class_annotation_list(C, 'grok.foo', None)
  [5, 7, 8]
  >>> util.class_annotation_list(C2, 'grok.foo', None)
  [5, 7, 9]

"""
import grok
from martian import util

class A(object):
    __grok_foo__ = [5]

class B(A):
    __grok_foo__ = [7]

class B2(A):
    pass

class C(B, B2):
    __grok_foo__ = [8]

class C2(B2, B):
    __grok_foo__ = [9]



"""
Inline templates that are not associated with a view class will
provoke an error:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  ...
  ConfigurationExecutionError: martian.error.GrokError: Found the following unassociated template(s) when grokking
  'grok.tests.view.inline_unassociated': club.  Define view classes inheriting
  from grok.View to enable the template(s).
  in:
  
"""
import grok

class Mammoth(grok.Model):
    pass

club = grok.PageTemplate("""\
<html><body><h1>GROK CLUB MAMMOTH!</h1></body></html>
""")
